@@ -1,5 +1,4 @@
 open Stdune
-open Dune_config
 open Dune_config_file
 module Console = Dune_console
 module Graph = Dune_graph.Graph
@@ -14,18 +13,52 @@ end
 
 open struct
   open Dune_rules
-  module Package = Package
   module Colors = Colors
   module Only_packages = Only_packages
 end
 
+module Workspace = Source.Workspace
+
 open struct
   open Cmdliner
   module Cmd = Cmd
-  module Term = Term
+
+  module Term = struct
+    include Term
+
+    (* Evaluate a parser, passing it no command-line arguments, in an
+       environment with no variables set. It's only valid to pass a parser
+       which can be evaluated with no arguments, otherwise a code error will
+       be raised. Returns the result of the parser. This is intended to be
+       used to extract the default value for a type implied by the behaviour
+       of its parser when no command-line arguments are passed to it. *)
+    let eval_no_args_empty_env t =
+      let raise_code_error data =
+        Code_error.raise "Unexpected result evaluating term with no args" data
+      in
+      (* Cmdliner doesn't allow argv to be empty. *)
+      let argv = [| "dune" |] in
+      let env _ = None in
+      match Cmd.eval_value ~argv ~env (Cmd.v (Cmd.info "dune") t) with
+      | Ok (`Ok x) -> x
+      | Ok `Help -> raise_code_error [ "ok", Dyn.string "help" ]
+      | Ok `Version -> raise_code_error [ "ok", Dyn.string "version" ]
+      | Error e ->
+        let error_string =
+          match e with
+          | `Parse -> "parse"
+          | `Term -> "term"
+          | `Exn -> "exn"
+        in
+        raise_code_error [ "error", Dyn.string error_string ]
+    ;;
+  end
+
   module Manpage = Manpage
 end
 
+module Package = Dune_lang.Package
+
 module Let_syntax = struct
   let ( let+ ) t f = Term.(const f $ t)
   let ( and+ ) a b = Term.(const (fun x y -> x, y) $ a $ b)
@@ -242,7 +275,6 @@ module Options_implied_by_dash_p = struct
       ; "--always-show-command-line"
       ; "--promote-install-files"
       ; "--require-dune-project-file"
-      ; "--ignore-lock-dir"
       ; "--default-target"
       ; "@install"
       ]
@@ -294,7 +326,7 @@ module Options_implied_by_dash_p = struct
     Term.with_used_args
       Arg.(
         value
-        & alias_opt (fun s -> [ "--release"; "--only-packages"; s ])
+        & alias_opt (fun s -> [ "--release"; "--ignore-lock-dir"; "--only-packages"; s ])
         & info
             [ "p"; "for-release-of-packages" ]
             ~docs
@@ -411,12 +443,12 @@ let shared_with_config_file =
     let doc =
       Printf.sprintf
         "Enable or disable Dune cache (%s). Default is `%s'."
-        (Arg.doc_alts_enum Config.Toggle.all)
-        (Config.Toggle.to_string Dune_config.default.cache_enabled)
+        (Arg.doc_alts_enum Dune_config.Cache.Toggle.all)
+        (Dune_config.Cache.Toggle.to_string Dune_config.default.cache_enabled)
     in
     Arg.(
       value
-      & opt (some (enum Config.Toggle.all)) None
+      & opt (some (enum Dune_config.Cache.Toggle.all)) None
       & info [ "cache" ] ~docs ~env:(Cmd.Env.info ~doc "DUNE_CACHE") ~doc)
   and+ cache_storage_mode =
     let doc =
@@ -487,6 +519,8 @@ let shared_with_config_file =
   ; cache_storage_mode
   ; action_stdout_on_success
   ; action_stderr_on_success
+  ; project_defaults = None
+  ; pkg_enabled = None
   ; experimental = None
   }
 ;;
@@ -552,14 +586,6 @@ let cache_debug_flags_term : Cache_debug_flags.t Term.t =
   value initial
 ;;
 
-module Action_runner = struct
-  type t =
-    | No
-    | Yes of
-        (Dune_lang.Dep_conf.t Dune_rpc_impl.Server.t
-         -> (Dune_engine.Action_exec.input -> Dune_engine.Action_runner.t option) Staged.t)
-end
-
 module Builder = struct
   type t =
     { debug_dep_path : bool
@@ -567,6 +593,7 @@ module Builder = struct
     ; debug_artifact_substitution : bool
     ; debug_load_dir : bool
     ; debug_digests : bool
+    ; debug_package_logs : bool
     ; wait_for_filesystem_clock : bool
     ; only_packages : Only_packages.Clflags.t
     ; capture_outputs : bool
@@ -587,7 +614,7 @@ module Builder = struct
     ; always_show_command_line : bool
     ; promote_install_files : bool
     ; file_watcher : Dune_engine.Scheduler.Run.file_watcher
-    ; workspace_config : Dune_rules.Workspace.Clflags.t
+    ; workspace_config : Workspace.Clflags.t
     ; cache_debug_flags : Dune_engine.Cache_debug_flags.t
     ; report_errors_config : Dune_engine.Report_errors_config.t
     ; separate_error_messages : bool
@@ -595,20 +622,19 @@ module Builder = struct
     ; require_dune_project_file : bool
     ; watch_exclusions : string list
     ; build_dir : string
-    ; store_digest_preimage : bool
     ; root : string option
     ; stats_trace_file : string option
     ; stats_trace_extended : bool
     ; allow_builds : bool
     ; default_root_is_cwd : bool
-    ; action_runner : Action_runner.t
     ; log_file : Dune_util.Log.File.t
     }
 
+  let root t = t.root
   let set_root t root = { t with root = Some root }
   let forbid_builds t = { t with allow_builds = false; no_print_directory = true }
+  let default_root_is_cwd t = t.default_root_is_cwd
   let set_default_root_is_cwd t x = { t with default_root_is_cwd = x }
-  let set_action_runner t x = { t with action_runner = x }
   let set_log_file t x = { t with log_file = x }
   let disable_log_file t = { t with log_file = No_log_file }
   let set_promote t v = { t with promote = Some v }
@@ -667,19 +693,16 @@ module Builder = struct
             [ "debug-digests" ]
             ~docs
             ~doc:"Explain why Dune decides to re-digest some files")
-    and+ store_digest_preimage =
+    and+ debug_package_logs =
+      let doc = "Always print the standard logs when building packages" in
       Arg.(
         value
         & flag
         & info
-            [ "debug-store-digest-preimage" ]
+            [ "debug-package-logs" ]
             ~docs
-            ~doc:
-              "Store digest preimage for all computed digests, so that it's possible to \
-               reverse them later, for debugging. The digests are stored in the shared \
-               cache (see --cache flag) as values, even if cache is otherwise disabled. \
-               This should be used only for debugging, since it's slow and it litters \
-               the shared cache.")
+            ~doc
+            ~env:(Cmd.Env.info ~doc "DUNE_DEBUG_PACKAGE_LOGS"))
     and+ no_buffer =
       let doc =
         "Do not buffer the output of commands executed by dune. By default dune buffers \
@@ -984,6 +1007,7 @@ module Builder = struct
     ; debug_artifact_substitution
     ; debug_load_dir
     ; debug_digests
+    ; debug_package_logs
     ; wait_for_filesystem_clock
     ; only_packages
     ; capture_outputs = not no_buffer
@@ -1025,16 +1049,102 @@ module Builder = struct
     ; require_dune_project_file
     ; watch_exclusions
     ; build_dir = Option.value ~default:default_build_dir build_dir
-    ; store_digest_preimage
     ; root
     ; stats_trace_file
     ; stats_trace_extended
     ; allow_builds = true
     ; default_root_is_cwd = false
-    ; action_runner = No
     ; log_file = Default
     }
   ;;
+
+  let default = Term.eval_no_args_empty_env term
+
+  let equal
+        t
+        { debug_dep_path
+        ; debug_backtraces
+        ; debug_artifact_substitution
+        ; debug_load_dir
+        ; debug_digests
+        ; debug_package_logs
+        ; wait_for_filesystem_clock
+        ; only_packages
+        ; capture_outputs
+        ; diff_command
+        ; promote
+        ; ignore_promoted_rules
+        ; force
+        ; no_print_directory
+        ; ignore_lock_dir
+        ; store_orig_src_dir
+        ; default_target
+        ; watch
+        ; print_metrics
+        ; dump_memo_graph_file
+        ; dump_memo_graph_format
+        ; dump_memo_graph_with_timing
+        ; dump_gc_stats
+        ; always_show_command_line
+        ; promote_install_files
+        ; file_watcher
+        ; workspace_config
+        ; cache_debug_flags
+        ; report_errors_config
+        ; separate_error_messages
+        ; stop_on_first_error
+        ; require_dune_project_file
+        ; watch_exclusions
+        ; build_dir
+        ; root
+        ; stats_trace_file
+        ; stats_trace_extended
+        ; allow_builds
+        ; default_root_is_cwd
+        ; log_file
+        }
+    =
+    Bool.equal t.debug_dep_path debug_dep_path
+    && Bool.equal t.debug_backtraces debug_backtraces
+    && Bool.equal t.debug_artifact_substitution debug_artifact_substitution
+    && Bool.equal t.debug_load_dir debug_load_dir
+    && Bool.equal t.debug_digests debug_digests
+    && Bool.equal t.debug_package_logs debug_package_logs
+    && Bool.equal t.wait_for_filesystem_clock wait_for_filesystem_clock
+    && Only_packages.Clflags.equal t.only_packages only_packages
+    && Bool.equal t.capture_outputs capture_outputs
+    && Option.equal String.equal t.diff_command diff_command
+    && Option.equal Dune_engine.Clflags.Promote.equal t.promote promote
+    && Bool.equal t.ignore_promoted_rules ignore_promoted_rules
+    && Bool.equal t.force force
+    && Bool.equal t.no_print_directory no_print_directory
+    && Bool.equal t.ignore_lock_dir ignore_lock_dir
+    && Bool.equal t.store_orig_src_dir store_orig_src_dir
+    && Arg.Dep.equal t.default_target default_target
+    && Dune_rpc_impl.Watch_mode_config.equal t.watch watch
+    && Bool.equal t.print_metrics print_metrics
+    && Option.equal Path.External.equal t.dump_memo_graph_file dump_memo_graph_file
+    && Graph.File_format.equal t.dump_memo_graph_format dump_memo_graph_format
+    && Bool.equal t.dump_memo_graph_with_timing dump_memo_graph_with_timing
+    && Option.equal Path.External.equal t.dump_gc_stats dump_gc_stats
+    && Bool.equal t.always_show_command_line always_show_command_line
+    && Bool.equal t.promote_install_files promote_install_files
+    && Dune_engine.Scheduler.Run.file_watcher_equal t.file_watcher file_watcher
+    && Source.Workspace.Clflags.equal t.workspace_config workspace_config
+    && Dune_engine.Cache_debug_flags.equal t.cache_debug_flags cache_debug_flags
+    && Dune_engine.Report_errors_config.equal t.report_errors_config report_errors_config
+    && Bool.equal t.separate_error_messages separate_error_messages
+    && Bool.equal t.stop_on_first_error stop_on_first_error
+    && Bool.equal t.require_dune_project_file require_dune_project_file
+    && List.equal String.equal t.watch_exclusions watch_exclusions
+    && String.equal t.build_dir build_dir
+    && Option.equal String.equal t.root root
+    && Option.equal String.equal t.stats_trace_file stats_trace_file
+    && Bool.equal t.stats_trace_extended stats_trace_extended
+    && Bool.equal t.allow_builds allow_builds
+    && Bool.equal t.default_root_is_cwd default_root_is_cwd
+    && Log.File.equal t.log_file log_file
+  ;;
 end
 
 type t =
@@ -1062,14 +1172,6 @@ let rpc t =
   | `Allow rpc -> `Allow (Lazy.force rpc)
 ;;
 
-let signal_watcher t =
-  match t.rpc with
-  | `Allow _ -> `Yes
-  | `Forbid_builds ->
-    (* if we aren't building anything, then we don't mind interrupting dune immediately *)
-    `No
-;;
-
 let watch_exclusions t = t.builder.watch_exclusions
 let stats t = t.stats
 
@@ -1141,7 +1243,7 @@ let print_entering_message c =
    we should probably refactor that at some point. *)
 let build (builder : Builder.t) =
   let root =
-    Workspace_root.create
+    Workspace_root.create_exn
       ~default_is_cwd:builder.default_root_is_cwd
       ~specified_by_user:builder.root
   in
@@ -1170,23 +1272,36 @@ let build (builder : Builder.t) =
              | Yes Passive -> Some 1.0
              | _ -> None
            in
-           let action_runner = Dune_engine.Action_runner.Rpc_server.create () in
            Dune_rpc_impl.Server.create
              ~lock_timeout
              ~registry
              ~root:root.dir
              ~handle:Dune_rules_rpc.register
-             ~watch_mode_config:builder.watch
-             ~parse_build:Dune_rules_rpc.parse_build
-             stats
-             action_runner))
+             ~parse_build_arg:Dune_rules_rpc.parse_build_arg
+             stats))
     else `Forbid_builds
   in
-  if builder.store_digest_preimage then Dune_engine.Reversible_digest.enable ();
   if builder.print_metrics then Dune_metrics.enable ();
   { builder; root; rpc; stats }
 ;;
 
+let maybe_init_cache (cache_config : Dune_cache.Config.t) =
+  match cache_config with
+  | Disabled -> cache_config
+  | Enabled _ ->
+    (match Dune_cache_storage.Layout.create_cache_directories () with
+     | Ok () -> cache_config
+     | Error (path, exn) ->
+       User_warning.emit
+         ~hints:
+           [ Pp.textf "Make sure the directory %s can be created" (Path.to_string path) ]
+         [ Pp.textf
+             "Cache directories could not be created: %s; disabling cache"
+             (Unix.error_message exn)
+         ];
+       Disabled)
+;;
+
 let init (builder : Builder.t) =
   let c = build builder in
   if c.root.dir <> Filename.current_dir_name then Sys.chdir c.root.dir;
@@ -1198,12 +1313,10 @@ let init (builder : Builder.t) =
   (* We need to print this before reading the workspace file, so that the editor
      can interpret errors in the workspace file. *)
   print_entering_message c;
-  Dune_rules.Workspace.Clflags.set c.builder.workspace_config;
+  Workspace.Clflags.set c.builder.workspace_config;
   let config =
     (* Here we make the assumption that this computation doesn't yield. *)
-    Fiber.run
-      (Memo.run (Dune_rules.Workspace.workspace_config ()))
-      ~iter:(fun () -> assert false)
+    Fiber.run (Memo.run (Workspace.workspace_config ())) ~iter:(fun () -> assert false)
   in
   let config =
     Dune_config.adapt_display
@@ -1218,38 +1331,33 @@ let init (builder : Builder.t) =
        | Yes _ -> true);
   Dune_engine.Execution_parameters.init
     (let open Memo.O in
-     let+ w = Dune_rules.Workspace.workspace () in
+     let+ w = Workspace.workspace () in
      Dune_engine.Execution_parameters.builtin_default
-     |> Dune_rules.Workspace.update_execution_parameters w);
+     |> Workspace.update_execution_parameters w);
   Dune_rules.Global.init ~capture_outputs:c.builder.capture_outputs;
   let cache_config =
     match config.cache_enabled with
-    | `Disabled -> Dune_cache.Config.Disabled
-    | `Enabled ->
+    | Disabled -> Dune_cache.Config.Disabled
+    | Enabled_except_user_rules | Enabled ->
       Enabled
         { storage_mode = Option.value config.cache_storage_mode ~default:Hardlink
         ; reproducibility_check = config.cache_reproducibility_check
         }
   in
-  Log.info [ Pp.textf "Shared cache: %s" (Config.Toggle.to_string config.cache_enabled) ];
+  Log.info
+    [ Pp.textf
+        "Shared cache: %s"
+        (Dune_config.Cache.Toggle.to_string config.cache_enabled)
+    ];
   Log.info
     [ Pp.textf
         "Shared cache location: %s"
         (Path.to_string Dune_cache_storage.Layout.root_dir)
     ];
-  let action_runner =
-    match builder.action_runner with
-    | No -> None
-    | Yes f ->
-      (match rpc c with
-       | `Forbid_builds -> Code_error.raise "action runners require building" []
-       | `Allow server -> Some (Staged.unstage @@ f server))
-  in
   Dune_rules.Main.init
-    ?action_runner
     ~stats:c.stats
     ~sandboxing_preference:config.sandboxing_preference
-    ~cache_config
+    ~cache_config:(maybe_init_cache cache_config)
     ~cache_debug_flags:c.builder.cache_debug_flags
     ();
   Only_packages.Clflags.set c.builder.only_packages;
@@ -1258,11 +1366,12 @@ let init (builder : Builder.t) =
   Dune_engine.Clflags.debug_backtraces c.builder.debug_backtraces;
   Dune_rules.Clflags.debug_artifact_substitution := c.builder.debug_artifact_substitution;
   Dune_engine.Clflags.debug_load_dir := c.builder.debug_load_dir;
-  Dune_engine.Clflags.debug_digests := c.builder.debug_digests;
   Dune_engine.Clflags.debug_fs_cache := c.builder.cache_debug_flags.fs_cache;
-  Dune_engine.Clflags.wait_for_filesystem_clock := c.builder.wait_for_filesystem_clock;
+  Dune_digest.Clflags.debug_digests := c.builder.debug_digests;
+  Dune_rules.Clflags.debug_package_logs := c.builder.debug_package_logs;
+  Dune_digest.Clflags.wait_for_filesystem_clock := c.builder.wait_for_filesystem_clock;
   Dune_engine.Clflags.capture_outputs := c.builder.capture_outputs;
-  Dune_engine.Clflags.diff_command := c.builder.diff_command;
+  Promote.Clflags.diff_command := c.builder.diff_command;
   Dune_engine.Clflags.promote := c.builder.promote;
   Dune_engine.Clflags.force := c.builder.force;
   Dune_engine.Clflags.stop_on_first_error := c.builder.stop_on_first_error;
@@ -1271,8 +1380,12 @@ let init (builder : Builder.t) =
   Dune_engine.Clflags.always_show_command_line := c.builder.always_show_command_line;
   Dune_rules.Clflags.ignore_promoted_rules := c.builder.ignore_promoted_rules;
   Dune_rules.Clflags.ignore_lock_dir := c.builder.ignore_lock_dir;
-  Dune_rules.Clflags.on_missing_dune_project_file
+  Source.Clflags.on_missing_dune_project_file
   := if c.builder.require_dune_project_file then Error else Warn;
+  (Dune_engine.Clflags.can_go_in_shared_cache_default
+   := match config.cache_enabled with
+      | Disabled | Enabled_except_user_rules -> false
+      | Enabled -> true);
   Log.info
     [ Pp.textf
         "Workspace root: %s"
