@@ -3,7 +3,7 @@ open Import
 let term =
   let+ builder = Common.Builder.term in
   let common, config = Common.init builder in
-  Scheduler.go ~common ~config
+  Scheduler.go_with_rpc_server ~common ~config
   @@ fun () ->
   let open Fiber.O in
   let* setup = Import.Main.setup () in
