@@ -5,7 +5,7 @@ let term =
   and+ context_name = Common.context_arg ~doc:"Build context to use."
   and+ format = Describe_format.arg in
   let common, config = Common.init builder in
-  Scheduler.go ~common ~config
+  Scheduler.go_with_rpc_server ~common ~config
   @@ fun () ->
   let open Fiber.O in
   let* setup = Import.Main.setup () in
