@@ -180,11 +180,12 @@ module Init_context = struct
   let make path defaults =
     let open Memo.O in
     let+ project =
-      (* CR-rgrinberg: why not get the project from the source tree? *)
+      (* CR-someday rgrinberg: why not get the project from the source tree? *)
       Dune_project.load
         ~dir:Path.Source.root
         ~files:Filename.Set.empty
         ~infer_from_opam_files:true
+        ~load_opam_file_with_contents:Dune_pkg.Opam_file.load_opam_file_with_contents
       >>| function
       | Some p -> p
       | None ->
@@ -245,20 +246,27 @@ module Component = struct
     module Common = struct
       type t =
         { name : Dune_lang.Atom.t
+        ; public : Public_name.t option
         ; libraries : Dune_lang.Atom.t list
         ; pps : Dune_lang.Atom.t list
         }
+
+      let package_name common =
+        let name =
+          match common.public with
+          | None -> Dune_lang.Atom.to_string common.name
+          | Some public -> Public_name.to_string public
+        in
+        Package.Name.of_string name
+      ;;
     end
 
     module Executable = struct
-      type t = { public : Public_name.t option }
+      type t = unit
     end
 
     module Library = struct
-      type t =
-        { public : Public_name.t option
-        ; inline_tests : bool
-        }
+      type t = { inline_tests : bool }
     end
 
     module Project = struct
@@ -355,11 +363,11 @@ module Component = struct
 
     let public_name_field = Encoder.field_o "public_name" Public_name.encode
 
-    let executable (common : Options.Common.t) (options : Options.Executable.t) =
-      make "executable" common [ public_name_field options.public ]
+    let executable (common : Options.Common.t) (() : Options.Executable.t) =
+      make "executable" common [ public_name_field common.public ]
     ;;
 
-    let library (common : Options.Common.t) { Options.Library.inline_tests; public } =
+    let library (common : Options.Common.t) { Options.Library.inline_tests } =
       check_module_name common.name;
       let common =
         if inline_tests
@@ -370,22 +378,25 @@ module Component = struct
           { common with pps })
         else common
       in
-      make "library" common [ public_name_field public; Field.inline_tests inline_tests ]
+      make
+        "library"
+        common
+        [ public_name_field common.public; Field.inline_tests inline_tests ]
     ;;
 
     let test common (() : Options.Test.t) = make "test" common []
 
     (* A list of CSTs for dune-project file content *)
     let dune_project
-      ~opam_file_gen
-      ~(defaults : Dune_config_file.Dune_config.Project_defaults.t)
-      dir
-      (common : Options.Common.t)
+          ~opam_file_gen
+          ~(defaults : Dune_config_file.Dune_config.Project_defaults.t)
+          dir
+          (common : Options.Common.t)
       =
       let cst =
         let package =
           Package.create
-            ~name:(Package.Name.of_string (Atom.to_string common.name))
+            ~name:(Options.Common.package_name common)
             ~loc:Loc.none
             ~version:None
             ~conflicts:[]
@@ -499,8 +510,8 @@ module Component = struct
       let lib_target =
         src
           { context = { context with dir = Path.relative dir "lib" }
-          ; options = { public = None; inline_tests = options.inline_tests }
-          ; common
+          ; options = { inline_tests = options.inline_tests }
+          ; common = { common with public = None }
           }
       in
       let test_target =
@@ -516,7 +527,7 @@ module Component = struct
         let libraries = Stanza_cst.add_to_list_set common.name common.libraries in
         bin
           { context = { context with dir = Path.relative dir "bin" }
-          ; options = { public = Some (Public_name.of_name_exn common.name) }
+          ; options = ()
           ; common = { common with libraries; name = Dune_lang.Atom.of_string "main" }
           }
       in
@@ -527,10 +538,7 @@ module Component = struct
       let lib_target =
         src
           { context = { context with dir = Path.relative dir "lib" }
-          ; options =
-              { public = Some (Public_name.of_name_exn common.name)
-              ; inline_tests = options.inline_tests
-              }
+          ; options = { inline_tests = options.inline_tests }
           ; common
           }
       in
@@ -548,13 +556,11 @@ module Component = struct
     let proj ({ common; options; _ } as opts : Options.Project.t Options.t) =
       let ({ template; pkg; _ } : Options.Project.t) = options in
       let dir = Path.Source.root in
-      let name =
-        Package.Name.parse_string_exn (Loc.none, Dune_lang.Atom.to_string common.name)
-      in
       let proj_target =
         let package_files =
           match (pkg : Options.Project.Pkg.t) with
           | Opam ->
+            let name = Options.Common.package_name common in
             let opam_file = Path.source @@ Package_name.file name ~dir in
             [ File.make_text (Path.parent_exn opam_file) (Path.basename opam_file) "" ]
           | Esy -> [ File.make_text (Path.source dir) "package.json" "" ]
