@@ -1,4 +1,5 @@
 open! Import
+open Pkg_common
 module Package_universe = Dune_pkg.Package_universe
 module Lock_dir = Dune_pkg.Lock_dir
 module Opam_repo = Dune_pkg.Opam_repo
@@ -29,21 +30,26 @@ let enumerate_lock_dirs_by_path ~lock_dirs () =
 
 let validate_lock_dirs ~lock_dirs () =
   let open Fiber.O in
-  let+ lock_dirs_by_path, local_packages =
+  let* lock_dirs_by_path, local_packages =
     Memo.both (enumerate_lock_dirs_by_path ~lock_dirs ()) Pkg_common.find_local_packages
     |> Memo.run
   in
   if List.is_empty lock_dirs_by_path
-  then Console.print [ Pp.text "No lockdirs to validate." ]
-  else (
-    match
-      List.filter_map lock_dirs_by_path ~f:(function
-        | Error e -> Some e
-        | Ok (path, lock_dir) ->
-          (match Package_universe.create local_packages lock_dir with
+  then
+    let+ () = Fiber.return () in
+    Console.print [ Pp.text "No lockdirs to validate." ]
+  else
+    let+ universes =
+      Fiber.parallel_map lock_dirs_by_path ~f:(function
+        | Error e -> Fiber.return (Some e)
+        | Ok (lock_dir_path, lock_dir) ->
+          let+ platform = solver_env_from_system_and_context ~lock_dir_path in
+          (match Package_universe.create ~platform local_packages lock_dir with
            | Ok _ -> None
-           | Error e -> Some (path, `Lock_dir_out_of_sync e)))
-    with
+           | Error e -> Some (lock_dir_path, `Lock_dir_out_of_sync e)))
+      >>| List.filter_opt
+    in
+    match universes with
     | [] -> ()
     | errors_by_path ->
       List.iter errors_by_path ~f:(fun (path, error) ->
@@ -68,7 +74,7 @@ let validate_lock_dirs ~lock_dirs () =
         [ Pp.text "Some lockdirs do not contain solutions for local packages:"
         ; Pp.enumerate errors_by_path ~f:(fun (path, _) ->
             Pp.text (Path.Source.to_string path))
-        ])
+        ]
 ;;
 
 let term =
@@ -76,7 +82,7 @@ let term =
   and+ lock_dirs = Pkg_common.Lock_dirs_arg.term in
   let builder = Common.Builder.forbid_builds builder in
   let common, config = Common.init builder in
-  Scheduler.go ~common ~config @@ validate_lock_dirs ~lock_dirs
+  Scheduler.go_with_rpc_server ~common ~config @@ validate_lock_dirs ~lock_dirs
 ;;
 
 let command = Cmd.v info term
