@@ -25,14 +25,14 @@ let dump sctx ~dir =
   and+ menhir_dump =
     Dune_rules.Menhir_rules.menhir_env ~dir
     |> Action_builder.of_memo
-    >>= Dune_rules.Menhir_env.dump
+    >>= Dune_lang.Menhir_env.dump
   and+ coq_dump = Dune_rules.Coq.Coq_rules.coq_env ~dir >>| Dune_rules.Coq.Coq_flags.dump
   and+ jsoo_js_dump =
-    let module Js_of_ocaml = Dune_rules.Js_of_ocaml in
+    let module Js_of_ocaml = Dune_lang.Js_of_ocaml in
     let* jsoo = Action_builder.of_memo (Dune_rules.Jsoo_rules.jsoo_env ~dir ~mode:JS) in
     Js_of_ocaml.Flags.dump ~mode:JS jsoo.flags
   and+ jsoo_wasm_dump =
-    let module Js_of_ocaml = Dune_rules.Js_of_ocaml in
+    let module Js_of_ocaml = Dune_lang.Js_of_ocaml in
     let* jsoo = Action_builder.of_memo (Dune_rules.Jsoo_rules.jsoo_env ~dir ~mode:Wasm) in
     Js_of_ocaml.Flags.dump ~mode:Wasm jsoo.flags
   in
@@ -85,7 +85,7 @@ let term =
              multiple fields.")
   in
   let common, config = Common.init builder in
-  Scheduler.go ~common ~config (fun () ->
+  Scheduler.go_with_rpc_server ~common ~config (fun () ->
     let open Fiber.O in
     let* setup = Import.Main.setup () in
     let* setup = Memo.run setup in
