@@ -1,5 +1,4 @@
 open Stdune
-open Dune_rules
 
 module Kind = struct
   type t =
@@ -18,9 +17,9 @@ module Kind = struct
   let lowest_priority = max_int
 
   let of_dir_contents files =
-    if String.Set.mem files Workspace.filename
+    if String.Set.mem files Source.Workspace.filename
     then Some Dune_workspace
-    else if Filename.Set.mem files Dune_project.filename
+    else if Filename.Set.mem files Dune_lang.Dune_project.filename
     then Some Dune_project
     else None
   ;;
@@ -88,7 +87,7 @@ let create ~default_is_cwd ~specified_by_user =
     | Some dn -> Some { Candidate.kind = Explicit; dir = dn; to_cwd = [] }
     | None ->
       let cwd = { Candidate.kind = Cwd; dir = "."; to_cwd = [] } in
-      if Dune_util.Execution_env.inside_dune
+      if Execution_env.inside_dune
       then Some cwd
       else (
         match find () with
