@@ -1,42 +1,38 @@
 (** {2 Command line} *)
 
-let concurrency, verbose, _keep_generated_files, debug, secondary, force_byte_compilation =
+let concurrency, verbose, debug, secondary, force_byte_compilation, static, build_dir =
+  let build_dir = ref "_boot" in
   let anon s = raise (Arg.Bad (Printf.sprintf "don't know what to do with %s\n" s)) in
   let concurrency = ref None in
   let verbose = ref false in
-  let keep_generated_files = ref false in
   let prog = Filename.basename Sys.argv.(0) in
   let debug = ref false in
   let secondary = ref false in
   let force_byte_compilation = ref false in
+  let static = ref false in
   Arg.parse
     [ "-j", Int (fun n -> concurrency := Some n), "JOBS Concurrency"
     ; "--verbose", Set verbose, " Set the display mode"
-    ; "--keep-generated-files", Set keep_generated_files, " Keep generated files"
+    ; "--keep-generated-files", Unit ignore, " Keep generated files"
     ; "--debug", Set debug, " Enable various debugging options"
     ; "--secondary", Set secondary, " Use the secondary compiler installation"
     ; ( "--force-byte-compilation"
       , Set force_byte_compilation
       , " Force bytecode compilation even if ocamlopt is available" )
+    ; "--static", Set static, " Build a static binary"
+    ; "--boot-dir", Set_string build_dir, " Set the boot directory"
     ]
     anon
     (Printf.sprintf "Usage: %s <options>\nOptions are:" prog);
-  ( !concurrency
-  , !verbose
-  , !keep_generated_files
-  , !debug
-  , !secondary
-  , !force_byte_compilation )
+  !concurrency, !verbose, !debug, !secondary, !force_byte_compilation, !static, !build_dir
 ;;
 
 (** {2 General configuration} *)
 
-let build_dir = "_boot"
-
 type task =
   { target : string * string
   ; external_libraries : string list
-  ; local_libraries : (string * string option * bool * string option) list
+  ; local_libraries : Libs.library list
   }
 
 let task =
@@ -50,12 +46,45 @@ let task =
 
 open StdLabels
 open Printf
-module StringSet = Set.Make (String)
-module StringMap = Map.Make (String)
+
+module String = struct
+  include String
+  module Set = Set.Make (String)
+  module Map = Map.Make (String)
+
+  let is_suffix t ~suffix = Filename.check_suffix t suffix
+
+  let is_prefix t ~prefix =
+    let len_s = length t
+    and len_pre = length prefix in
+    let rec aux i =
+      if i = len_pre
+      then true
+      else if unsafe_get t i <> unsafe_get prefix i
+      then false
+      else aux (i + 1)
+    in
+    len_s >= len_pre && aux 0
+  ;;
+end
 
 module List = struct
   include List
 
+  let partition_map_skip t ~f =
+    let rec loop l m r = function
+      | [] -> l, m, r
+      | x :: xs ->
+        (match f x with
+         | `Skip -> loop l m r xs
+         | `Left x -> loop (x :: l) m r xs
+         | `Middle x -> loop l (x :: m) r xs
+         | `Right x -> loop l m (x :: r) xs)
+    in
+    let l, m, r = loop [] [] [] t in
+    rev l, rev m, rev r
+  ;;
+
   let rec filter_map l ~f =
     match l with
     | [] -> []
@@ -71,8 +100,8 @@ let ( ^/ ) = Filename.concat
 let fatal fmt =
   ksprintf
     (fun s ->
-      prerr_endline s;
-      exit 2)
+       prerr_endline s;
+       exit 2)
     fmt
 ;;
 
@@ -99,6 +128,7 @@ module Status_line = struct
       displayed := new_displayed)
   ;;
 
+  let clear () = Printf.printf "\r*s\r%!"
   let () = at_exit (fun () -> Printf.printf "\r%*s\r" (String.length !displayed) "")
 end
 
@@ -162,20 +192,29 @@ let split_lines s =
   loop ~acc:[] 0 0 ~last_is_cr:false
 ;;
 
-(* copy a file - fails if the file exists *)
-let copy ?(header = "") directive a b =
+let do_then_copy ~f a b =
   if Sys.file_exists b then fatal "%s already exists" b;
   let ic = open_in_bin a in
   let len = in_channel_length ic in
   let s = really_input_string ic len in
   close_in ic;
   let oc = open_out_bin b in
-  output_string oc header;
-  fprintf oc "#%s 1 %S\n" directive a;
+  f oc;
   output_string oc s;
   close_out oc
 ;;
 
+(* copy a file - fails if the file exists *)
+let copy a b = do_then_copy ~f:(fun _ -> ()) a b
+
+(* copy a file and insert a header - fails if the file exists *)
+let copy_with_header ~header a b = do_then_copy ~f:(fun oc -> output_string oc header) a b
+
+(* copy a file and insert a directive - fails if the file exists *)
+let copy_with_directive ~directive a b =
+  do_then_copy ~f:(fun oc -> fprintf oc "#%s 1 %S\n" directive a) a b
+;;
+
 let path_sep = if Sys.win32 then ';' else ':'
 
 let split_path s =
@@ -496,6 +535,7 @@ end = struct
           | Some x -> sprintf "cd %s && %s" x cmdline
           | None -> cmdline
         in
+        Status_line.clear ();
         prerr_endline cmdline;
         prerr_string stderr_s;
         flush stderr);
@@ -568,7 +608,7 @@ module Config : sig
   val ocamlyacc : string
   val mode : Mode.t
   val ocaml_archive_ext : string
-  val ocaml_config : unit -> string StringMap.t Fiber.t
+  val ocaml_config : unit -> string String.Map.t Fiber.t
   val output_complete_obj_arg : string
   val unix_library_flags : string list
 end = struct
@@ -626,9 +666,9 @@ end = struct
   let ocaml_config () =
     Process.run_and_capture ocamlc [ "-config" ]
     >>| fun s ->
-    List.fold_left (split_lines s) ~init:StringMap.empty ~f:(fun acc line ->
+    List.fold_left (split_lines s) ~init:String.Map.empty ~f:(fun acc line ->
       match Scanf.sscanf line "%[^:]: %s" (fun k v -> k, v) with
-      | k, v -> StringMap.add k v acc
+      | k, v -> String.Map.add k v acc
       | exception _ ->
         fatal "invalid line in output of 'ocamlc -config': %s" (String.escaped line))
   ;;
@@ -720,7 +760,7 @@ module Build_info = struct
        | Some v -> sprintf "Some %S" v);
     pr "\n";
     let libs =
-      List.map task.local_libraries ~f:(fun (name, _, _, _) -> name, "version")
+      List.map task.local_libraries ~f:(fun (lib : Libs.library) -> lib.path, "version")
       @ List.map task.external_libraries ~f:(fun name ->
         name, {|Some "[distributed with OCaml]"|})
       |> List.sort ~cmp:(fun (a, _) (b, _) -> String.compare a b)
@@ -735,35 +775,98 @@ end
 
 module Library = struct
   module File_kind = struct
+    type asm =
+      { syntax : [ `Gas | `Intel ]
+      ; arch : [ `Amd64 ] option
+      ; os : [ `Win | `Unix ] option
+      ; assembler : [ `C_comp | `Msvc_asm ]
+      }
+
+    type c =
+      { arch : [ `Arm64 | `X86 ] option
+      ; flags : string list
+      }
+
     type t =
       | Header
-      | C
+      | C of c
+      | Asm of asm
       | Ml
       | Mli
       | Mll
       | Mly
 
-    let analyse fn =
-      let dn = Filename.dirname fn in
-      let fn = Filename.basename fn in
+    let analyse file =
+      let fn = Filename.basename file in
       let i =
         try String.index fn '.' with
         | Not_found -> String.length fn
       in
       match String.sub fn ~pos:i ~len:(String.length fn - i) with
+      | (".S" | ".asm") as ext ->
+        let syntax = if ext = ".S" then `Gas else `Intel in
+        let os, arch, assembler =
+          let fn = Filename.remove_extension fn in
+          let check suffix = String.is_suffix fn ~suffix in
+          if check "x86-64_unix"
+          then Some `Unix, Some `Amd64, `C_comp
+          else if check "x86-64_windows_gnu"
+          then Some `Win, Some `Amd64, `C_comp
+          else if check "x86-64_windows_msvc"
+          then Some `Win, Some `Amd64, `Msvc_asm
+          else None, None, `C_comp
+        in
+        Some (Asm { syntax; arch; os; assembler })
+      | ".c" ->
+        let arch, flags =
+          let fn = Filename.remove_extension fn in
+          let check suffix = String.is_suffix fn ~suffix in
+          let x86 gnu _msvc =
+            (* CR rgrinberg: select msvc flags on windows *)
+            Some `X86, gnu
+          in
+          if check "_sse2"
+          then x86 [ "-msse2" ] [ "/arch:SSE2" ]
+          else if check "_sse41"
+          then x86 [ "-msse4.1" ] [ "/arch:AVX" ]
+          else if check "_avx2"
+          then x86 [ "-mavx2" ] [ "/arch:AVX2" ]
+          else if check "_avx512"
+          then x86 [ "-mavx512f"; "-mavx512vl"; "-mavx512bw" ] [ "/arch:AVX512" ]
+          else if String.is_suffix fn ~suffix:"_neon"
+          then Some `Arm64, []
+          else None, []
+        in
+        Some (C { arch; flags })
       | ".h" -> Some Header
-      | ".c" -> Some C
       | ".ml" -> Some Ml
       | ".mli" -> Some Mli
       | ".mll" -> Some Mll
       | ".mly" -> Some Mly
       | ".defaults.ml" ->
         let fn' = String.sub fn ~pos:0 ~len:i ^ ".ml" in
+        let dn = Filename.dirname file in
         if Sys.file_exists (dn ^/ fn') then None else Some Ml
       | _ -> None
     ;;
   end
 
+  type source =
+    { file : string
+    ; kind : File_kind.t
+    }
+
+  type c_file =
+    { name : string
+    ; flags : string list
+    }
+
+  type asm_file =
+    { assembler : [ `C_comp | `Msvc_asm ]
+    ; flags : string list
+    ; out_file : string
+    }
+
   module Wrapper = struct
     type t =
       { toplevel_module : string
@@ -775,27 +878,26 @@ module Library = struct
       | None -> None
       | Some namespace ->
         let namespace = String.capitalize_ascii namespace in
-        if StringSet.equal modules (StringSet.singleton namespace)
+        if String.Set.equal modules (String.Set.singleton namespace)
         then None
-        else if StringSet.mem namespace modules
+        else if String.Set.mem namespace modules
         then Some { toplevel_module = namespace; alias_module = namespace ^ "__" }
         else Some { toplevel_module = namespace; alias_module = namespace }
     ;;
 
-    let mangle_filename t fn (kind : File_kind.t) =
-      let base =
-        let fn = Filename.basename fn in
-        String.sub fn ~pos:0 ~len:(String.index fn '.') |> String.uncapitalize_ascii
-      in
+    let mangle_filename t ({ file; kind } : source) =
+      let base = Filename.basename file in
       match kind with
-      | C -> base ^ ".c"
-      | Header -> base ^ ".h"
-      | _ ->
+      | Asm _ | C _ | Header -> base
+      | Mll | Mly | Ml | Mli ->
         let ext =
           match kind with
           | Mli -> ".mli"
           | _ -> ".ml"
         in
+        let base =
+          String.sub base ~pos:0 ~len:(String.index base '.') |> String.uncapitalize_ascii
+        in
         (match t with
          | None -> base ^ ext
          | Some t ->
@@ -818,10 +920,10 @@ module Library = struct
       | Some t ->
         let fn = String.uncapitalize_ascii t.alias_module ^ ".ml" in
         let oc = open_out (build_dir ^/ fn) in
-        StringSet.iter
+        String.Set.iter
           (fun m ->
-            if m <> t.toplevel_module
-            then fprintf oc "module %s = %s__%s\n" m t.toplevel_module m)
+             if m <> t.toplevel_module
+             then fprintf oc "module %s = %s__%s\n" m t.toplevel_module m)
           modules;
         close_out oc;
         Some fn
@@ -839,7 +941,7 @@ module Library = struct
           then if scan_subdirs then loop (readdir file) acc else acc
           else (
             match File_kind.analyse file with
-            | Some kind -> (file, kind) :: acc
+            | Some kind -> { file; kind } :: acc
             | None -> acc)
         in
         loop files acc
@@ -847,63 +949,158 @@ module Library = struct
     loop (readdir dir) []
   ;;
 
-  let process (dir, namespace, scan_subdirs, build_info_module) =
+  type t =
+    { ocaml_files : string list
+    ; alias_file : string option
+    ; c_files : c_file list
+    ; asm_files : asm_file list
+    }
+
+  let keep_asm
+        { File_kind.syntax; arch; os; assembler = _ }
+        ~ccomp_type
+        ~architecture
+        ~os_type
+    =
+    (match os with
+     | Some `Unix -> String.equal os_type "Unix"
+     | Some `Win -> String.equal os_type "Win32"
+     | None -> true)
+    && (match syntax, ccomp_type with
+        | `Intel, "msvc" -> true
+        | `Gas, "msvc" -> false
+        | `Gas, _ -> true
+        | `Intel, _ -> false)
+    &&
+    match arch, architecture with
+    | None, _ -> true
+    | Some `Amd64, "amd64" -> true
+    | Some `Amd64, _ -> false
+  ;;
+
+  let keep_c { File_kind.arch; flags = _ } ~architecture =
+    match arch with
+    | None -> true
+    | Some `Arm64 -> architecture = "arm64"
+    | Some `X86 -> architecture = "amd64" || architecture = "x86_64"
+  ;;
+
+  let process
+        { Libs.path = dir
+        ; main_module_name = namespace
+        ; include_subdirs_unqualified = scan_subdirs
+        ; special_builtin_support = build_info_module
+        }
+        ~ocaml_config
+        ~word_size
+        ~os_type
+    =
     let files = scan ~dir ~scan_subdirs in
     let modules =
-      List.fold_left files ~init:StringSet.empty ~f:(fun acc (fn, kind) ->
-        match (kind : File_kind.t) with
-        | Header | C -> acc
-        | Ml | Mli | Mll | Mly ->
-          let module_name =
-            let fn = Filename.basename fn in
-            String.sub fn ~pos:0 ~len:(String.index fn '.') |> String.capitalize_ascii
-          in
-          StringSet.add module_name acc)
-    in
-    let modules =
+      let modules =
+        List.fold_left files ~init:String.Set.empty ~f:(fun acc { file = fn; kind } ->
+          match (kind : File_kind.t) with
+          | Asm _ | Header | C _ -> acc
+          | Ml | Mli | Mll | Mly ->
+            let module_name =
+              let fn = Filename.basename fn in
+              String.sub fn ~pos:0 ~len:(String.index fn '.') |> String.capitalize_ascii
+            in
+            String.Set.add module_name acc)
+      in
       match build_info_module with
       | None -> modules
-      | Some m -> StringSet.add (String.capitalize_ascii m) modules
+      | Some m -> String.Set.add (String.capitalize_ascii m) modules
     in
     let wrapper = Wrapper.make ~namespace ~modules in
     let header = Wrapper.header wrapper in
     Fiber.fork_and_join
       (fun () ->
-        Fiber.parallel_map files ~f:(fun (fn, kind) ->
-          let mangled = Wrapper.mangle_filename wrapper fn kind in
-          let dst = build_dir ^/ mangled in
-          match kind with
-          | Header | C ->
-            copy "line" fn dst;
-            Fiber.return [ mangled ]
-          | Ml | Mli ->
-            copy "" fn dst ~header;
-            Fiber.return [ mangled ]
-          | Mll -> copy_lexer fn dst ~header >>> Fiber.return [ mangled ]
-          | Mly -> copy_parser fn dst ~header >>> Fiber.return [ mangled; mangled ^ "i" ]))
+         Fiber.parallel_map files ~f:(fun ({ file = fn; kind } as source) ->
+           let mangled = Wrapper.mangle_filename wrapper source in
+           let dst = build_dir ^/ mangled in
+           (match kind with
+            | Asm _ ->
+              copy fn dst;
+              Fiber.return [ mangled ]
+            | Header | C _ ->
+              copy_with_directive ~directive:"line" fn dst;
+              Fiber.return [ mangled ]
+            | Ml | Mli ->
+              copy_with_header ~header fn dst;
+              Fiber.return [ mangled ]
+            | Mll -> copy_lexer fn dst ~header >>> Fiber.return [ mangled ]
+            | Mly ->
+              (* CR rgrinberg: what if the parser already has an mli? *)
+              copy_parser fn dst ~header >>> Fiber.return [ mangled; mangled ^ "i" ])
+           >>| function
+           | mangled -> List.map mangled ~f:(fun m -> source, m)))
       (fun () ->
-        match build_info_module with
-        | None -> Fiber.return None
-        | Some m ->
-          let fn = String.uncapitalize_ascii m ^ ".ml" in
-          let mangled = Wrapper.mangle_filename wrapper fn Ml in
-          let oc = open_out (build_dir ^/ mangled) in
-          Build_info.gen_data_module oc
-          >>| fun () ->
-          close_out oc;
-          Some mangled)
+         match build_info_module with
+         | None -> Fiber.return None
+         | Some m ->
+           let src =
+             let fn = String.uncapitalize_ascii m ^ ".ml" in
+             { file = fn; kind = Ml }
+           in
+           let mangled = Wrapper.mangle_filename wrapper src in
+           let oc = open_out (build_dir ^/ mangled) in
+           Build_info.gen_data_module oc
+           >>| fun () ->
+           close_out oc;
+           Some (src, mangled))
     >>| fun (files, build_info_file) ->
-    let files = List.concat files in
-    let files =
-      match build_info_file with
-      | None -> files
-      | Some fn -> fn :: files
-    in
     let alias_file = Wrapper.generate_wrapper wrapper modules in
-    let c_files, ocaml_files =
-      List.partition files ~f:(fun fn -> Filename.extension fn = ".c")
+    let c_files, ocaml_files, asm_files =
+      let files =
+        let files = List.concat files in
+        match build_info_file with
+        | None -> files
+        | Some fn -> fn :: files
+      in
+      let ext_obj =
+        try String.Map.find "ext_obj" ocaml_config with
+        | Not_found -> ".o"
+      in
+      let ccomp_type = String.Map.find "ccomp_type" ocaml_config in
+      let architecture = String.Map.find "architecture" ocaml_config in
+      List.partition_map_skip files ~f:(fun (src, fn) ->
+        match src.kind with
+        | C c ->
+          if keep_c c ~architecture
+          then (
+            let extra_flags =
+              if String.is_prefix ~prefix:"blake3_" fn
+              then
+                if String.equal os_type "Cygwin" || String.equal word_size "32"
+                then
+                  [ "-DBLAKE3_NO_SSE2"
+                  ; "-DBLAKE3_NO_SSE41"
+                  ; "-DBLAKE3_NO_AVX2"
+                  ; "-DBLAKE3_NO_AVX512"
+                  ]
+                else []
+              else []
+            in
+            `Left { flags = extra_flags @ c.flags; name = fn })
+          else `Skip
+        | Ml | Mli | Mly | Mll -> `Middle fn
+        | Header -> `Skip
+        | Asm asm ->
+          if keep_asm asm ~ccomp_type ~architecture ~os_type
+          then (
+            let out_file = Filename.chop_extension fn ^ ext_obj in
+            `Right
+              { flags =
+                  (match asm.assembler with
+                   | `C_comp -> [ "-c"; fn; "-o"; out_file ]
+                   | `Msvc_asm -> [ "/nologo"; "/quiet"; "/Fo" ^ out_file; "/c"; fn ])
+              ; assembler = asm.assembler
+              ; out_file
+              })
+          else `Skip)
     in
-    ocaml_files, alias_file, c_files
+    { ocaml_files; alias_file; c_files; asm_files }
   ;;
 end
 
@@ -935,25 +1132,25 @@ let convert_dependencies ~all_source_files (file, dependencies) =
     if filename = Filename.chop_extension file
     then (* Self-reference *)
       None
-    else if StringSet.mem (filename ^ ".mli") all_source_files
+    else if String.Set.mem (filename ^ ".mli") all_source_files
     then
-      if (not is_mli) && StringSet.mem (filename ^ ".ml") all_source_files
+      if (not is_mli) && String.Set.mem (filename ^ ".ml") all_source_files
       then
         (* We need to build the .ml for inlining info *)
         Some [ filename ^ ".mli"; filename ^ ".ml" ]
       else (* .mli files never depend on .ml files *)
         Some [ filename ^ ".mli" ]
-    else if StringSet.mem (filename ^ ".ml") all_source_files
+    else if String.Set.mem (filename ^ ".ml") all_source_files
     then
       (* If there's no .mli, then we must always depend on the .ml *)
       Some [ filename ^ ".ml" ]
     else (* This is a module coming from an external library *)
       None
   in
-  let dependencies = List.concat (List.filter_map ~f:convert_module dependencies) in
-  (* .ml depends on .mli, if it exists *)
   let dependencies =
-    if (not is_mli) && StringSet.mem (file ^ "i") all_source_files
+    let dependencies = List.concat (List.filter_map ~f:convert_module dependencies) in
+    (* .ml depends on .mli, if it exists *)
+    if (not is_mli) && String.Set.mem (file ^ "i") all_source_files
     then (file ^ "i") :: dependencies
     else dependencies
   in
@@ -968,25 +1165,29 @@ let write_args file args =
 
 let get_dependencies libraries =
   let alias_files =
-    List.fold_left libraries ~init:[] ~f:(fun acc (_, alias_file, _) ->
-      match alias_file with
+    List.fold_left libraries ~init:[] ~f:(fun acc (lib : Library.t) ->
+      match lib.alias_file with
       | None -> acc
       | Some fn -> fn :: acc)
   in
-  let all_source_files = List.map ~f:(fun (x, _, _) -> x) libraries |> List.concat in
+  let all_source_files =
+    List.map ~f:(fun (lib : Library.t) -> lib.ocaml_files) libraries |> List.concat
+  in
   write_args "source_files" all_source_files;
   ocamldep (mk_flags "-map" alias_files @ [ "-args"; "source_files" ])
   >>| fun dependencies ->
   let all_source_files =
     List.fold_left
       alias_files
-      ~init:(StringSet.of_list all_source_files)
-      ~f:(fun acc fn -> StringSet.add fn acc)
+      ~init:(String.Set.of_list all_source_files)
+      ~f:(fun acc fn -> String.Set.add fn acc)
   in
   let deps =
     List.rev_append
       ((* Alias files have no dependencies *)
-       List.rev_map alias_files ~f:(fun fn -> fn, []))
+       List.rev_map
+         alias_files
+         ~f:(fun fn -> fn, []))
       (List.rev_map dependencies ~f:(convert_dependencies ~all_source_files))
   in
   if debug
@@ -998,20 +1199,27 @@ let get_dependencies libraries =
   deps
 ;;
 
-let assemble_libraries { local_libraries; target = _, main; _ } =
-  let libraries =
-    (* In order to assemble all the sources in one place, the executables
+let assemble_libraries
+      { local_libraries; target = _, main; _ }
+      ~ocaml_config
+      ~word_size
+      ~os_type
+  =
+  (* In order to assemble all the sources in one place, the executables
        modules are also put in a namespace *)
-    let task_lib =
-      let dir = Filename.dirname main in
-      let namespace =
-        String.capitalize_ascii (Filename.chop_extension (Filename.basename main))
-      in
-      dir, Some namespace, true (* enable (include_subdirs unqualified *), None
+  let task_lib =
+    let dir = Filename.dirname main in
+    let namespace =
+      String.capitalize_ascii (Filename.chop_extension (Filename.basename main))
     in
-    local_libraries @ [ task_lib ]
+    { Libs.path = dir
+    ; main_module_name = Some namespace
+    ; include_subdirs_unqualified = true
+    ; special_builtin_support = None
+    }
   in
-  Fiber.parallel_map ~f:Library.process libraries
+  local_libraries @ [ task_lib ]
+  |> Fiber.parallel_map ~f:(Library.process ~ocaml_config ~word_size ~os_type)
 ;;
 
 type status =
@@ -1026,23 +1234,21 @@ let resolve_externals external_libraries =
       | "unix" -> "unix" ^ Config.ocaml_archive_ext, Config.unix_library_flags
       | s -> fatal "unhandled external library %s" s
     in
-    let externals = List.map ~f:convert external_libraries in
-    List.split externals
+    List.map ~f:convert external_libraries |> List.split
   in
   let external_includes = List.concat external_includes in
   external_libraries, external_includes
 ;;
 
 let sort_files dependencies ~main =
-  let n = List.length dependencies in
-  let deps_by_file = Hashtbl.create n in
+  let deps_by_file = Hashtbl.create (List.length dependencies) in
   List.iter dependencies ~f:(fun (file, deps) -> Hashtbl.add deps_by_file file deps);
-  let seen = ref StringSet.empty in
+  let seen = ref String.Set.empty in
   let res = ref [] in
   let rec loop file =
-    if not (StringSet.mem file !seen)
+    if not (String.Set.mem file !seen)
     then (
-      seen := StringSet.add file !seen;
+      seen := String.Set.add file !seen;
       List.iter (Hashtbl.find deps_by_file file) ~f:loop;
       res := file :: !res)
   in
@@ -1052,7 +1258,7 @@ let sort_files dependencies ~main =
 
 let common_build_args name ~external_includes ~external_libraries =
   List.concat
-    [ [ "-o"; Filename.concat "../_boot" (name ^ ".exe"); "-g" ]
+    [ [ "-o"; name ^ ".exe"; "-g" ]
     ; (match Config.mode with
        | Byte -> [ Config.output_complete_obj_arg ]
        | Native -> [])
@@ -1063,18 +1269,46 @@ let common_build_args name ~external_includes ~external_libraries =
 
 let allow_unstable_sources = [ "-alert"; "-unstable" ]
 
+let ocaml_warnings =
+  let warnings =
+    [ (* Warning 49 [no-cmi-file]: no cmi file was found in path for module *)
+      "-49"
+    ; (* Warning 23: all the fields are explicitly listed in this record: the
+        'with' clause is useless. 
+
+         In order to stay version independent, we use a trick with `with` by
+         creating a dummy value and filling in the fields available in every
+         OCaml version. forced_major_collections is the one missing in versions
+         older than 4.12. We therefore disable warning 23 for our purposes. *)
+      "-23"
+    ; (* Warning 53 [misplaced-attribute]: the "alert" attribute cannot appear
+        in this context
+
+        Any .mli files that begin wtih [@@@alert] will cause the compiler to
+        emit warning 53 due to the way we alias modules with an `open!`. It may
+        be possible to use the command line flag `-open` instead, however this
+        complicates dependency tracking so disabling this warning instead is
+        suitable for our purposes. *)
+      "-53"
+    ]
+  in
+  [ "-w"; String.concat ~sep:"" warnings ]
+;;
+
 let build
-  ~ocaml_config
-  ~dependencies
-  ~c_files
-  ~link_flags
-  { target = name, main; external_libraries; _ }
+      ~ocaml_config
+      ~dependencies
+      ~c_files
+      ~asm_files
+      ~build_flags
+      ~link_flags
+      { target = name, main; external_libraries; _ }
   =
+  let c_compiler = String.Map.find "c_compiler" ocaml_config in
   let ext_obj =
-    try StringMap.find "ext_obj" ocaml_config with
+    try String.Map.find "ext_obj" ocaml_config with
     | Not_found -> ".o"
   in
-  let external_libraries, external_includes = resolve_externals external_libraries in
   let num_dependencies = List.length dependencies in
   let table = Hashtbl.create num_dependencies in
   Status_line.num_jobs := num_dependencies;
@@ -1090,6 +1324,7 @@ let build
     | Started fut -> Fiber.Future.wait fut
     | exception Not_found -> fatal "file not found: %s" m
   in
+  let external_libraries, external_includes = resolve_externals external_libraries in
   List.iter dependencies ~f:(fun (file, deps) ->
     Hashtbl.add
       table
@@ -1102,7 +1337,8 @@ let build
              ~cwd:build_dir
              Config.compiler
              (List.concat
-                [ [ "-c"; "-g"; "-no-alias-deps"; "-w"; "-49-6" ]
+                [ [ "-c"; "-g"; "-no-alias-deps" ]
+                ; ocaml_warnings
                 ; allow_unstable_sources
                 ; external_includes
                 ; [ file ]
@@ -1110,19 +1346,34 @@ let build
   Fiber.fork_and_join_unit
     (fun () -> build (Filename.basename main))
     (fun () ->
-      Fiber.parallel_map c_files ~f:(fun file ->
-        Process.run
-          ~cwd:build_dir
-          Config.compiler
-          (List.concat [ [ "-c"; "-g" ]; external_includes; [ file ] ])
-        >>| fun () -> Filename.chop_extension file ^ ext_obj))
+       (Fiber.fork_and_join (fun () ->
+          Fiber.parallel_map c_files ~f:(fun { Library.name = file; flags } ->
+            let flags =
+              List.map flags ~f:(fun flag -> [ "-ccopt"; flag ]) |> List.concat
+            in
+            Process.run
+              ~cwd:build_dir
+              Config.compiler
+              (List.concat
+                 [ [ "-c"; "-g" ]; external_includes; build_flags; [ file ]; flags ])
+            >>| fun () -> Filename.chop_extension file ^ ext_obj)))
+         (fun () ->
+            Fiber.parallel_map asm_files ~f:(fun { Library.assembler; flags; out_file } ->
+              Process.run
+                ~cwd:build_dir
+                (match assembler with
+                 | `C_comp -> c_compiler
+                 | `Msvc_asm -> "ml64.exe")
+                flags
+              >>| fun () -> out_file))
+       >>| fun (x, y) -> x @ y)
   >>= fun obj_files ->
-  let compiled_ml_ext =
-    match Config.mode with
-    | Byte -> ".cmo"
-    | Native -> ".cmx"
-  in
   let compiled_ml_files =
+    let compiled_ml_ext =
+      match Config.mode with
+      | Byte -> ".cmo"
+      | Native -> ".cmx"
+    in
     List.filter_map (sort_files dependencies ~main) ~f:(fun fn ->
       match Filename.extension fn with
       | ".ml" -> Some (Filename.remove_extension fn ^ compiled_ml_ext)
@@ -1132,11 +1383,13 @@ let build
   Process.run
     ~cwd:build_dir
     Config.compiler
-    (List.concat
+    (let static_flags = if static then [ "-ccopt"; "-static" ] else [] in
+     List.concat
        [ common_build_args name ~external_includes ~external_libraries
        ; obj_files
        ; [ "-args"; "compiled_ml_files" ]
        ; link_flags
+       ; static_flags
        ; allow_unstable_sources
        ])
 ;;
@@ -1151,6 +1404,11 @@ let rec rm_rf fn =
 
 and clear dir = List.iter (readdir dir) ~f:rm_rf
 
+let rec get_flags system = function
+  | (set, f) :: r -> if List.mem system ~set then f else get_flags system r
+  | [] -> []
+;;
+
 (** {2 Bootstrap process} *)
 let main () =
   (try clear build_dir with
@@ -1159,20 +1417,26 @@ let main () =
    | Unix.Unix_error (Unix.EEXIST, _, _) -> ());
   Config.ocaml_config ()
   >>= fun ocaml_config ->
-  assemble_libraries task
+  let word_size = String.Map.find "word_size" ocaml_config in
+  let os_type = String.Map.find "os_type" ocaml_config in
+  assemble_libraries ~ocaml_config ~word_size ~os_type task
   >>= fun libraries ->
-  let c_files = List.map ~f:(fun (_, _, c_files) -> c_files) libraries |> List.concat in
+  let c_files =
+    List.map ~f:(fun (lib : Library.t) -> lib.c_files) libraries |> List.concat
+  in
+  let asm_files =
+    List.map ~f:(fun (lib : Library.t) -> lib.asm_files) libraries |> List.concat
+  in
   get_dependencies libraries
   >>= fun dependencies ->
-  let link_flags =
-    match StringMap.find_opt "system" ocaml_config with
+  let ocaml_system =
+    match String.Map.find_opt "system" ocaml_config with
     | None -> assert false
-    | Some platform ->
-      (match List.assoc_opt platform Libs.link_flags with
-       | None -> []
-       | Some flags -> flags)
+    | Some s -> s
   in
-  build ~ocaml_config ~dependencies ~c_files ~link_flags task
+  let build_flags = get_flags ocaml_system Libs.build_flags in
+  let link_flags = get_flags ocaml_system Libs.link_flags in
+  build ~ocaml_config ~dependencies ~asm_files ~c_files ~build_flags ~link_flags task
 ;;
 
 let () = Fiber.run (main ())
