@@ -9,7 +9,8 @@ let%expect_test _ =
   String.Map.of_list_multi [ "a", 1; "b", 1; "a", 2; "a", 3; "b", 2 ]
   |> String.Map.to_dyn (list int)
   |> print_dyn;
-  [%expect {|
+  [%expect
+    {|
 map { "a" : [ 1; 2; 3 ]; "b" : [ 1; 2 ] }
 |}]
 ;;
