@@ -10,140 +10,160 @@ let split s ~on = String.split s ~on |> list string |> print_dyn
 
 let%expect_test _ =
   take "foobar" 3;
-  [%expect {|
+  [%expect
+    {|
 "foo"
 |}]
 ;;
 
 let%expect_test _ =
   take "foobar" 0;
-  [%expect {|
+  [%expect
+    {|
 ""
 |}]
 ;;
 
 let%expect_test _ =
   take "foo" 10;
-  [%expect {|
+  [%expect
+    {|
 "foo"
 |}]
 ;;
 
 let%expect_test _ =
   take "" 10;
-  [%expect {|
+  [%expect
+    {|
 ""
 |}]
 ;;
 
 let%expect_test _ =
   take "" 0;
-  [%expect {|
+  [%expect
+    {|
 ""
 |}]
 ;;
 
 let%expect_test _ =
   drop "" 0;
-  [%expect {|
+  [%expect
+    {|
 ""
 |}]
 ;;
 
 let%expect_test _ =
   drop "foo" 0;
-  [%expect {|
+  [%expect
+    {|
 "foo"
 |}]
 ;;
 
 let%expect_test _ =
   drop "foo" 5;
-  [%expect {|
+  [%expect
+    {|
 ""
 |}]
 ;;
 
 let%expect_test _ =
   drop "foobar" 3;
-  [%expect {|
+  [%expect
+    {|
 "bar"
 |}]
 ;;
 
 let%expect_test _ =
   split_n "foobar" 3;
-  [%expect {|
+  [%expect
+    {|
 ("foo", "bar")
 |}]
 ;;
 
 let%expect_test _ =
   split_n "foobar" 10;
-  [%expect {|
+  [%expect
+    {|
 ("foobar", "")
 |}]
 ;;
 
 let%expect_test _ =
   split_n "foobar" 0;
-  [%expect {|
+  [%expect
+    {|
 ("", "foobar")
 |}]
 ;;
 
 let%expect_test _ =
   split_n "foobar" 6;
-  [%expect {|
+  [%expect
+    {|
 ("foobar", "")
 |}]
 ;;
 
 let%expect_test _ =
   split_n "" 0;
-  [%expect {|
+  [%expect
+    {|
 ("", "")
 |}]
 ;;
 
 let%expect_test _ =
   split_n "" 10;
-  [%expect {|
+  [%expect
+    {|
 ("", "")
 |}]
 ;;
 
 let%expect_test _ =
   String.longest_prefix [ "food"; "foo"; "foo-bar" ] |> string |> print_dyn;
-  [%expect {|
+  [%expect
+    {|
 "foo"
 |}]
 ;;
 
 let%expect_test _ =
   String.drop_suffix "foobar" ~suffix:"bar" |> option string |> print_dyn;
-  [%expect {|
+  [%expect
+    {|
 Some "foo"
 |}]
 ;;
 
 let%expect_test _ =
   String.drop_suffix "foobar" ~suffix:"foobar" |> option string |> print_dyn;
-  [%expect {|
+  [%expect
+    {|
 Some ""
 |}]
 ;;
 
 let%expect_test _ =
   String.drop_suffix "foobar" ~suffix:"" |> option string |> print_dyn;
-  [%expect {|
+  [%expect
+    {|
 Some "foobar"
 |}]
 ;;
 
 let%expect_test _ =
   String.drop_suffix "foobar" ~suffix:"foo" |> option string |> print_dyn;
-  [%expect {|
+  [%expect
+    {|
 None
 |}]
 ;;
