@@ -44,14 +44,11 @@ module type Ast = sig
     | Copy of path * target
     | Symlink of path * target
     | Hardlink of path * target
-    | System of string
     | Bash of string
     | Write_file of target * File_perm.t * string
     | Rename of target * target
     | Remove_tree of target
     | Mkdir of target
-    | Diff of (path, target) Diff.t
-    | Merge_files_into of path list * string list * target
     | Pipe of Outputs.t * t list
     | Extension of ext
 end
@@ -80,20 +77,18 @@ module type Helpers = sig
   val cat : path list -> t
   val copy : path -> target -> t
   val symlink : path -> target -> t
-  val system : string -> t
   val bash : string -> t
   val write_file : ?perm:File_perm.t -> target -> string -> t
   val rename : target -> target -> t
   val remove_tree : target -> t
   val mkdir : target -> t
-  val diff : ?optional:bool -> ?mode:Diff.Mode.t -> path -> target -> t
 end
 
-module Ext = struct
+module Exec = struct
   type context =
     { targets : Targets.Validated.t option
     ; context : Build_context.t option
-    ; purpose : Process.purpose
+    ; metadata : Process.metadata
     ; rule_loc : Loc.t
     ; build_deps : Dep.Set.t -> Dep.Facts.t Fiber.t
     }
@@ -104,22 +99,25 @@ module Ext = struct
     ; stdout_to : Process.Io.output Process.Io.t
     ; stderr_to : Process.Io.output Process.Io.t
     ; stdin_from : Process.Io.input Process.Io.t
+    ; prepared_dependencies : Dune_action_plugin.Private.Protocol.Dependency.Set.t
     ; exit_codes : int Predicate.t
     }
+end
 
+module Ext = struct
   module type Spec = sig
     type ('path, 'target) t
 
     val name : string
     val version : int
     val is_useful_to : memoize:bool -> bool
-    val encode : ('p, 't) t -> ('p -> Dune_sexp.t) -> ('t -> Dune_sexp.t) -> Dune_sexp.t
+    val encode : ('p, 't) t -> ('p -> Sexp.t) -> ('t -> Sexp.t) -> Sexp.t
     val bimap : ('a, 'b) t -> ('a -> 'x) -> ('b -> 'y) -> ('x, 'y) t
 
     val action
       :  (Path.t, Path.Build.t) t
-      -> ectx:context
-      -> eenv:env
+      -> ectx:Exec.context
+      -> eenv:Exec.env
       -> (* cwong: For now, I think we should only worry about extensions with
             known dependencies. In the future, we may generalize this to return
             an [Action_exec.done_or_more_deps], but that may be trickier to get
