@@ -34,19 +34,11 @@ module Make (Src : Action_intf.Ast) (Dst : Action_intf.Ast) = struct
     | Copy (x, y) -> Copy (f_path ~dir x, f_target ~dir y)
     | Symlink (x, y) -> Symlink (f_path ~dir x, f_target ~dir y)
     | Hardlink (x, y) -> Hardlink (f_path ~dir x, f_target ~dir y)
-    | System x -> System (f_string ~dir x)
     | Bash x -> Bash (f_string ~dir x)
     | Write_file (x, perm, y) -> Write_file (f_target ~dir x, perm, f_string ~dir y)
     | Rename (x, y) -> Rename (f_target ~dir x, f_target ~dir y)
     | Remove_tree x -> Remove_tree (f_target ~dir x)
     | Mkdir x -> Mkdir (f_target ~dir x)
-    | Diff ({ file1; file2; _ } as diff) ->
-      Diff { diff with file1 = f_path ~dir file1; file2 = f_target ~dir file2 }
-    | Merge_files_into (sources, extras, target) ->
-      Merge_files_into
-        ( List.map sources ~f:(f_path ~dir)
-        , List.map extras ~f:(f_string ~dir)
-        , f_target ~dir target )
     | Pipe (outputs, l) -> Pipe (outputs, List.map l ~f:(fun t -> f t ~dir))
     | Extension ext -> Extension (f_ext ~dir ext)
   ;;
