@@ -1,3 +1,5 @@
+open Import
+
 (** Controls when the errors are reported.
 
     - [Early] - report errors as soon as they are discovered. This means the
@@ -15,8 +17,14 @@ type t =
      scheduling. *)
   | Twice
 
+let equal a b =
+  match a, b with
+  | Early, Early | Deterministic, Deterministic | Twice, Twice -> true
+  | _, _ -> false
+;;
+
 let default : t =
-  match Dune_util.Execution_env.inside_dune with
+  match Execution_env.inside_dune with
   | true -> Deterministic
   | false -> Early
 ;;
