@@ -1,6 +1,6 @@
-open Stdune
-open Dune_sexp
+open Import
 open Dune_util.Action
+open Dune_sexp
 
 module Action_plugin = struct
   let syntax =
@@ -70,7 +70,7 @@ end
 
 module Env_update = struct
   module Op = struct
-    type t =
+    type t = OpamParserTypes.env_update_op =
       | Eq
       | PlusEq
       | EqPlus
@@ -115,9 +115,9 @@ module Env_update = struct
   let map t ~f = { t with value = f t.value }
 
   let equal
-    value_equal
-    { op; var; value }
-    { op = other_op; var = other_var; value = other_value }
+        value_equal
+        { op; var; value }
+        { op = other_op; var = other_var; value = other_value }
     =
     Op.equal op other_op
     && Ordering.is_eq (Env.Var.compare var other_var)
@@ -176,6 +176,7 @@ type t =
   | Substitute of String_with_vars.t * String_with_vars.t
   | Withenv of String_with_vars.t Env_update.t list * t
   | When of Slang.blang * t
+  | Format_dune_file of String_with_vars.t * String_with_vars.t
 
 let is_dev_null t = String_with_vars.is_pform t (Var Dev_null)
 
@@ -350,6 +351,11 @@ let cstrs_dune_file t =
     , Syntax.since Stanza.syntax (2, 7)
       >>> let+ script = sw in
           Cram script )
+  ; ( "format-dune-file"
+    , Syntax.since Stanza.syntax (3, 18)
+      >>> let+ src = sw
+          and+ dst = sw in
+          Format_dune_file (src, dst) )
   ]
 ;;
 
@@ -390,7 +396,7 @@ let decode_pkg =
             Withenv (ops, t) )
     ; ( "when"
       , Syntax.since Stanza.syntax (0, 1)
-        >>> let+ condition = Slang.decode_blang
+        >>> let+ condition = Slang.Blang.decode
             and+ action = t in
             When (condition, action) )
     ; ( "run"
@@ -457,7 +463,8 @@ let rec encode =
   | Withenv (ops, t) ->
     List [ atom "withenv"; List (List.map ~f:Env_update.encode ops); encode t ]
   | When (condition, action) ->
-    List [ atom "when"; Slang.encode_blang condition; encode action ]
+    List [ atom "when"; Slang.Blang.encode condition; encode action ]
+  | Format_dune_file (src, dst) -> List [ atom "format-dune-file"; sw src; sw dst ]
 ;;
 
 (* In [Action_exec] we rely on one-to-one mapping between the cwd-relative paths
@@ -495,7 +502,8 @@ let ensure_at_most_one_dynamic_run ~loc action =
     | Diff _
     | Substitute _
     | Patch _
-    | Cram _ -> false
+    | Cram _
+    | Format_dune_file _ -> false
     | Pipe (_, ts) | Progn ts | Concurrent ts ->
       List.fold_left ts ~init:false ~f:(fun acc t ->
         let have_dyn = loop t in
@@ -597,6 +605,7 @@ let rec map_string_with_vars t ~f =
     When
       ( blang_map_string_with_vars condition ~f:(slang_map_string_with_vars ~f)
       , map_string_with_vars t ~f )
+  | Format_dune_file (src, dst) -> Format_dune_file (f src, f dst)
 ;;
 
 let remove_locs = map_string_with_vars ~f:String_with_vars.remove_locs
