@@ -1,5 +1,5 @@
-open Stdune
-open Dune_util.Alias_name
+open Import
+open Alias_name
 
 let invalid_alias = Pp.textf "%S is not a valid alias name"
 
@@ -10,7 +10,7 @@ let decode =
     | None -> User_error.raise ~loc [ invalid_alias s ]
     | Some s -> s
   in
-  let open Dune_sexp.Decoder in
-  let* syntax = Dune_sexp.Syntax.get_exn Stanza.syntax in
+  let open Decoder in
+  let* syntax = Syntax.get_exn Stanza.syntax in
   plain_string (fun ~loc s -> parse_string_exn ~syntax (loc, s))
 ;;
