@@ -1,4 +1,4 @@
-open Stdune
+open Import
 
 (** Invariants: - Named s -> s <> "" and s does not contain '.' or '/' -
     Anonymous p -> p is a local path in the source tree *)
@@ -16,6 +16,6 @@ module Map : Map.S with type key = t
 
 val anonymous : Path.Source.t -> t
 val named : Loc.t -> string -> t
-val encode : t Dune_sexp.Encoder.t
-val decode : t Dune_sexp.Decoder.t
+val encode : t Encoder.t
+val decode : t Decoder.t
 val name : t -> string option
