@@ -1,4 +1,4 @@
-open Stdune
+open Import
 
 type t
 
@@ -6,8 +6,8 @@ val to_opam : t -> OpamVariable.t
 val of_opam : OpamVariable.t -> t
 val compare : t -> t -> Ordering.t
 val to_dyn : t -> Dyn.t
-val encode : t Dune_sexp.Encoder.t
-val decode : t Dune_sexp.Decoder.t
+val encode : t Encoder.t
+val decode : t Decoder.t
 val equal : t -> t -> bool
 
 module Map : Map.S with type key = t
@@ -23,6 +23,7 @@ val os_family : t
 val opam_version : t
 val with_test : t
 val with_doc : t
+val with_dev_setup : t
 val sys_ocaml_version : t
 val name : t
 val version : t
@@ -31,7 +32,11 @@ val build : t
 val dev : t
 val one_of : t -> t list -> bool
 
+(** The set of variable names whose values are expected to differ depending on
+    the current platform. *)
+val platform_specific : Set.t
+
 module Project : sig
-  val encode : t Dune_sexp.Encoder.t
-  val decode : t Dune_sexp.Decoder.t
+  val encode : t Encoder.t
+  val decode : t Decoder.t
 end
