@@ -1,4 +1,4 @@
-open! Stdune
+open Import
 
 type t
 
@@ -8,5 +8,7 @@ val of_string_user_error : Loc.t * string -> (t, User_message.t) result
 val to_string : t -> string
 val equal : t -> t -> bool
 val to_dyn : t -> Dyn.t
-val encode : t Dune_sexp.Encoder.t
-val decode : t Dune_sexp.Decoder.t
+val encode : t Encoder.t
+val decode : t Decoder.t
+
+include Comparable_intf.S with type key := t
