@@ -1,4 +1,5 @@
-open Dune_sexp.Decoder
+open Import
+open Decoder
 
 (* Can be extended later if needed *)
 type t = Toggle.t
@@ -8,12 +9,9 @@ let to_string = function
   | `Enabled -> "enabled"
 ;;
 
-let encode t = Dune_sexp.Encoder.string (to_string t)
+let encode t = Encoder.string (to_string t)
 let decoder = enum [ "disabled", `Disabled; "enabled", `Enabled ]
-
-let field =
-  field_o "subst" (Dune_sexp.Syntax.since Stanza.syntax (3, 0) >>> located decoder)
-;;
+let field = field_o "subst" (Syntax.since Stanza.syntax (3, 0) >>> located decoder)
 
 let of_config = function
   | None -> `Enabled
