@@ -1,4 +1,4 @@
-open Stdune
+open Import
 
 type t =
   | Public
@@ -12,22 +12,21 @@ let to_string = function
 let to_dyn t = Dyn.string (to_string t)
 
 let encode =
-  let open Dune_sexp.Encoder in
+  let open Encoder in
   function
   | Public -> string "public"
   | Private -> string "private"
 ;;
 
 let decode =
-  let open Dune_sexp.Decoder in
-  plain_string (fun ~loc ->
-       function
-       | "public" -> Public
-       | "private" -> Private
-       | _ ->
-         User_error.raise
-           ~loc
-           [ Pp.text "Not a valid visibility. Valid visibility is public or private" ])
+  let open Decoder in
+  plain_string (fun ~loc -> function
+    | "public" -> Public
+    | "private" -> Private
+    | _ ->
+      User_error.raise
+        ~loc
+        [ Pp.text "Not a valid visibility. Valid visibility is public or private" ])
 ;;
 
 module Map = struct
