@@ -21,7 +21,8 @@ module Local_package = Local_package
 module Package_universe = Package_universe
 module Variable_value = Variable_value
 module Resolved_package = Resolved_package
-module Pin_stanza = Pin_stanza
+module Pin = Pin
 module Package_name = Package_name
 module Ocamlformat = Ocamlformat
 module Dev_tool = Dev_tool
+module Outdated = Outdated
