@@ -1,17 +1,21 @@
 open! Import
 
-(** All of the packages in a dune project including local packages and packages
-    in a lockdir. The lockdir is guaranteed to contain a valid dependency
-    solution for the local packages. *)
+(** all of the packages in a dune project under the constraints of a given
+    platform, including local packages and packages in a lockdir. the lockdir
+    is guaranteed to contain a valid dependency solution for the local
+    packages. *)
 type t
 
+(** Create a universe for a given platform.
+    TODO: Consider changing this to cover all platforms simultaneously. *)
 val create
-  :  Local_package.t Package_name.Map.t
+  :  platform:Solver_env.t
+  -> Local_package.t Package_name.Map.t
   -> Lock_dir.t
   -> (t, User_message.t) result
 
 (** Verifies if the dependencies described in the project file are still
-    synchronize with the dependencies selected in the lock directroy. If it is
+    synchronized with the dependencies selected in the lock directroy. If it is
     not the case, it returns the hash of the new dependency set. *)
 val up_to_date
   :  Local_package.t Package_name.Map.t
