@@ -17,9 +17,19 @@ let true_ = "true"
 let false_ = "false"
 let string = Fun.id
 let equal = String.equal
+let compare = String.compare
 let to_dyn = Dyn.string
 let to_string = Fun.id
 let decode = Decoder.string
 let encode = Encoder.string
 let to_opam_filter t = OpamTypes.FString t
 let to_opam_variable_contents t = OpamTypes.S t
+
+let sentinel_value_of_variable_name variable_name =
+  let uppercase_replacing_dash_with_underscore =
+    Package_variable_name.to_string variable_name
+    |> String.uppercase
+    |> String.replace_char ~from:'-' ~to_:'_'
+  in
+  string (String.concat ~sep:"" [ "__"; uppercase_replacing_dash_with_underscore ])
+;;
