@@ -27,3 +27,5 @@ let pkg_install = Alias.Name.of_string "pkg-install"
 let ocaml_index = standard "ocaml-index"
 let runtest = standard "runtest"
 let all = standard "all"
+let default = standard "default"
+let empty = standard "empty"
