@@ -10,6 +10,17 @@ let flags flags =
   list (pair (list string) (list string)) flags
 ;;
 
+let type_def =
+  {|
+type library =
+  { path : string
+  ; main_module_name : string option
+  ; include_subdirs_unqualified : bool
+  ; special_builtin_support : string option
+  }
+|}
+;;
+
 let rule sctx ~requires_link =
   let open Action_builder.O in
   let* () = Action_builder.return () in
@@ -45,36 +56,40 @@ let rule sctx ~requires_link =
       in
       let open Memo.O in
       let+ is_multi_dir =
-        let+ dc = Dir_contents.get sctx ~dir in
-        match Dir_contents.dirs dc with
+        Dir_contents.get sctx ~dir
+        >>| Dir_contents.dirs
+        >>| function
         | _ :: _ :: _ -> true
         | _ -> false
       in
-      Dyn.Tuple
-        [ Path.Build.drop_build_context_exn dir
-          |> Path.Source.to_local
-          |> Path.Local.to_dyn
-        ; Dyn.option
-            Module_name.to_dyn
-            (match Lib_info.main_module_name info with
-             | From _ -> None
-             | This x -> x)
-        ; Dyn.Bool is_multi_dir
-        ; Dyn.option Module_name.to_dyn special_builtin_support
+      Dyn.record
+        [ ( "path"
+          , Path.Build.drop_build_context_exn dir
+            |> Path.Source.to_local
+            |> Path.Local.to_dyn )
+        ; ( "main_module_name"
+          , Dyn.option
+              Module_name.to_dyn
+              (match Lib_info.main_module_name info with
+               | From _ -> None
+               | This x -> x) )
+        ; "include_subdirs_unqualified", Dyn.Bool is_multi_dir
+        ; "special_builtin_support", Dyn.option Module_name.to_dyn special_builtin_support
         ])
     |> Action_builder.of_memo
   in
   let externals =
     List.filter_map externals ~f:(fun lib ->
       let name = Lib.name lib in
-      Option.some_if (Lib_name.equal name (Lib_name.of_string "threads.posix")) name)
+      Option.some_if (not (Lib_name.equal name (Lib_name.of_string "threads.posix"))) name)
   in
   Format.asprintf
     "%a@."
     Pp.to_fmt
     (Pp.concat
        ~sep:Pp.cut
-       [ def "external_libraries" (Dyn.list Lib_name.to_dyn externals)
+       [ Pp.verbatim type_def
+       ; def "external_libraries" (Dyn.list Lib_name.to_dyn externals)
        ; Pp.nop
        ; def "local_libraries" (List locals)
        ; Pp.nop
