@@ -1,7 +1,7 @@
 open Import
 open Memo.O
 
-let ocaml_flags t ~dir (spec : Ocaml_flags.Spec.t) =
+let ocaml_flags t ~dir (spec : Dune_lang.Ocaml_flags.Spec.t) =
   let* expander = Super_context.expander t ~dir in
   let* flags =
     let+ ocaml_flags = Ocaml_flags_db.ocaml_flags_env ~dir in
@@ -62,30 +62,55 @@ type kind =
       ; empty_module_interface_if_absent : bool
       }
 
+let fold_resolve (t : _ Preprocess.t) ~init ~f =
+  match t with
+  | Pps t -> Resolve.Memo.List.fold_left t.pps ~init ~f
+  | No_preprocessing | Action _ | Future_syntax _ -> Resolve.Memo.return init
+;;
+
+let instrumentation_deps t ~instrumentation_backend =
+  let open Resolve.Memo.O in
+  let f = function
+    | Preprocess.With_instrumentation.Ordinary _ -> Resolve.Memo.return []
+    | Instrumentation_backend { libname; deps; flags = _ } ->
+      instrumentation_backend libname
+      >>| (function
+       | Some _ -> deps
+       | None -> [])
+  in
+  Instrumentation.fold t ~init:[] ~f:(fun t init ->
+    let f acc t =
+      let+ x = f t in
+      x :: acc
+    in
+    fold_resolve t ~init ~f)
+  >>| List.rev
+  >>| List.flatten
+;;
+
 let modules_rules
-  ~preprocess
-  ~preprocessor_deps
-  ~lint
-  ~empty_module_interface_if_absent
-  sctx
-  expander
-  ~dir
-  scope
-  modules
-  ~lib_name
-  ~empty_intf_modules
+      ~preprocess
+      ~preprocessor_deps
+      ~lint
+      ~empty_module_interface_if_absent
+      sctx
+      expander
+      ~dir
+      scope
+      modules
+      ~lib_name
+      ~empty_intf_modules
   =
   let* pp =
     let instrumentation_backend = Lib.DB.instrumentation_backend (Scope.libs scope) in
     let* preprocess_with_instrumentation =
       (* TODO wrong and blocks loading all the rules in this directory *)
-      Resolve.Memo.read_memo
-        (Preprocess.Per_module.with_instrumentation preprocess ~instrumentation_backend)
+      Instrumentation.with_instrumentation preprocess ~instrumentation_backend
+      |> Resolve.Memo.read_memo
     in
     let* instrumentation_deps =
       (* TODO wrong and blocks loading all the rules in this directory *)
-      Resolve.Memo.read_memo
-        (Preprocess.Per_module.instrumentation_deps preprocess ~instrumentation_backend)
+      instrumentation_deps preprocess ~instrumentation_backend |> Resolve.Memo.read_memo
     in
     Pp_spec_rules.make
       sctx
