@@ -7,10 +7,3 @@ let debug_artifact_substitution = ref false
 let debug_package_logs = ref false
 let ignore_lock_dir = ref false
 let concurrency = ref 1
-
-type on_missing_dune_project_file =
-  | Error
-  | Warn
-  | Ignore
-
-let on_missing_dune_project_file = ref Warn
