@@ -22,12 +22,4 @@ val debug_package_logs : bool ref
 (** Whether we are ignoring "dune.lock/". *)
 val ignore_lock_dir : bool ref
 
-type on_missing_dune_project_file =
-  | Error
-  | Warn
-  | Ignore
-
-(** Desired behavior when dune project file is absent *)
-val on_missing_dune_project_file : on_missing_dune_project_file ref
-
 val concurrency : int ref
