@@ -96,7 +96,6 @@ type t =
   ; melange_package_name : Lib_name.t option
   ; modes : Lib_mode.Map.Set.t
   ; bin_annot : bool
-  ; ocamldep_modules_data : Ocamldep.Modules_data.t
   ; loc : Loc.t option
   ; ocaml : Ocaml_toolchain.t
   }
@@ -124,39 +123,37 @@ let vimpl t = t.vimpl
 let modes t = t.modes
 let bin_annot t = t.bin_annot
 let context t = Super_context.context t.super_context
-let ocamldep_modules_data t = t.ocamldep_modules_data
 let dep_graphs t = t.modules.dep_graphs
 let ocaml t = t.ocaml
 
 let create
-  ~super_context
-  ~scope
-  ~obj_dir
-  ~modules
-  ~flags
-  ~requires_compile
-  ~requires_link
-  ?(preprocessing = Pp_spec.dummy)
-  ~opaque
-  ?stdlib
-  ~js_of_ocaml
-  ~package
-  ~melange_package_name
-  ?vimpl
-  ?modes
-  ?bin_annot
-  ?loc
-  ()
+      ~super_context
+      ~scope
+      ~obj_dir
+      ~modules
+      ~flags
+      ~requires_compile
+      ~requires_link
+      ?(preprocessing = Pp_spec.dummy)
+      ~opaque
+      ?stdlib
+      ~js_of_ocaml
+      ~package
+      ~melange_package_name
+      ?vimpl
+      ?modes
+      ?bin_annot
+      ?loc
+      ()
   =
   let project = Scope.project scope in
   let context = Super_context.context super_context in
   let* ocaml = Context.ocaml context in
   let direct_requires, hidden_requires =
-    if Dune_project.implicit_transitive_deps project
-    then Memo.Lazy.force requires_link, Resolve.Memo.return []
-    else if Version.supports_hidden_includes ocaml.version
-            && Dune_project.dune_version project >= (3, 17)
-    then (
+    match Dune_project.implicit_transitive_deps project ocaml.version with
+    | Enabled -> Memo.Lazy.force requires_link, Resolve.Memo.return []
+    | Disabled -> requires_compile, Resolve.Memo.return []
+    | Disabled_with_hidden_includes ->
       let requires_hidden =
         let open Resolve.Memo.O in
         let+ requires_compile = requires_compile
@@ -165,8 +162,7 @@ let create
         List.iter ~f:(fun lib -> Table.set requires_table lib ()) requires_compile;
         List.filter requires_link ~f:(fun l -> not (Table.mem requires_table l))
       in
-      requires_compile, requires_hidden)
-    else requires_compile, Resolve.Memo.return []
+      requires_compile, requires_hidden
   in
   let sandbox = Sandbox_config.no_special_requirements in
   let modes =
@@ -181,17 +177,14 @@ let create
     let profile = Context.profile context in
     eval_opaque ocaml profile opaque
   in
-  let ocamldep_modules_data : Ocamldep.Modules_data.t =
-    { dir = Obj_dir.dir obj_dir
-    ; sandbox
-    ; obj_dir
-    ; sctx = super_context
-    ; vimpl
-    ; modules
-    ; stdlib
-    }
-  in
-  let+ dep_graphs = Dep_rules.rules ocamldep_modules_data
+  let+ dep_graphs =
+    Dep_rules.rules
+      ~dir:(Obj_dir.dir obj_dir)
+      ~sandbox
+      ~obj_dir
+      ~sctx:super_context
+      ~vimpl
+      ~modules
   and+ bin_annot =
     match bin_annot with
     | Some b -> Memo.return b
@@ -217,7 +210,6 @@ let create
   ; melange_package_name
   ; modes
   ; bin_annot
-  ; ocamldep_modules_data
   ; loc
   ; ocaml
   }
@@ -329,9 +321,8 @@ let entry_module_names sctx t =
   match Lib_info.entry_modules (Lib.info t) with
   | External d -> Resolve.Memo.of_result d
   | Local ->
-    let+ modules = Dir_contents.modules_of_lib sctx t in
-    let modules = Option.value_exn modules in
-    Resolve.return (Modules.With_vlib.entry_modules modules |> List.map ~f:Module.name)
+    let+ modules = Dir_contents.modules_of_local_lib sctx (Lib.Local.of_lib_exn t) in
+    modules |> Modules.entry_modules |> List.map ~f:Module.name |> Resolve.return
 ;;
 
 let root_module_entries t =
