@@ -15,12 +15,15 @@ type t =
 let decode ~coq_syntax =
   Dune_lang.Decoder.(
     enum'
-      [ ("vo", return VoOnly)
-      ; ("vos", return VosOnly)
+      [ "vo", return VoOnly
+      ; "vos", return VosOnly
       ; ( "native"
-        , Dune_sexp.Syntax.deprecated_in coq_syntax (0, 7)
+        , Dune_sexp.Syntax.deprecated_in
+            coq_syntax
+            (0, 7)
             ~extra_info:
-              "Since Coq lang 0.7 native mode is automatically inferred from \
-               the configuration of Coq."
+              "Since Coq lang 0.7 native mode is automatically inferred from the \
+               configuration of Coq."
           >>> return Native )
       ])
+;;
