@@ -8,7 +8,7 @@
 open Import
 
 (** A Coq Path is a non-dune source of Coq theories, these can come from Coq's
-    stdlib and user-contrib location, and [COQPATH] enviroment variable. *)
+    stdlib and user-contrib location, and [COQPATH] environment variable. *)
 
 (** This module is similar to [Dir_contents] but for globally installed libs *)
 
