@@ -8,7 +8,7 @@ open Coq_stanza
 type t =
   { libraries : Coq_module.t list Coq_lib_name.Map.t
   ; directories : Path.Build.t list Coq_lib_name.Map.t
-      (* [directories] is used to compute the include paths for Coq's native
+    (* [directories] is used to compute the include paths for Coq's native
          mode *)
   ; extract : Coq_module.t Loc.Map.t
   ; rev_map : [ `Theory of Theory.t | `Extraction of Extraction.t ] Coq_module.Map.t
@@ -28,11 +28,13 @@ let empty =
 ;;
 
 let coq_modules_of_files ~dirs =
-  let filter_v_files ({ Source_file_dir.dir = _; path_to_root = _; files } as sd) =
+  let filter_v_files
+        ({ Source_file_dir.dir = _; path_to_root = _; files; source_dir = _ } as sd)
+    =
     { sd with files = String.Set.filter files ~f:(fun f -> Filename.check_suffix f ".v") }
   in
   let dirs = List.map dirs ~f:filter_v_files in
-  let build_mod_dir { Source_file_dir.dir; path_to_root = prefix; files } =
+  let build_mod_dir { Source_file_dir.dir; path_to_root = prefix; files; source_dir = _ } =
     String.Set.to_list_map files ~f:(fun file ->
       let name, _ = Filename.split_extension file in
       let name = Coq_module.Name.make name in
