@@ -12,8 +12,10 @@ type t =
   ; enabled_if : Blang.t
   ; locks : Locks.t
   ; package : Package.t option
+  ; runtest_alias : (Loc.t * bool) option
+  ; timeout : (Loc.t * float) option
   }
 
-type Stanza.t += T of t
+include Stanza.S with type t := t
 
-val decode : t Dune_lang.Decoder.t
+val stanza : (string * Stanza.t list Dune_lang.Decoder.t) list
