@@ -9,7 +9,7 @@ let loc_of_dune_file st_dir =
      let open Option.O in
      let* dune_file = Source_tree.Dir.dune_file st_dir in
      (* TODO not really correct. we need to know the [(subdir ..)] that introduced this *)
-     Dune_file0.path dune_file
+     Source.Dune_file.path dune_file
    with
    | Some s -> s
    | None -> Path.Source.relative (Source_tree.Dir.path st_dir) "_unknown_")
@@ -22,9 +22,10 @@ type t =
   ; dir : Path.Build.t
   ; text_files : Filename.Set.t
   ; foreign_sources : Foreign_sources.t Memo.Lazy.t
-  ; mlds : (Documentation.t * Path.Build.t list) list Memo.Lazy.t
+  ; mlds : (Documentation.t * Doc_sources.mld list) list Memo.Lazy.t
   ; coq : Coq_sources.t Memo.Lazy.t
   ; ml : Ml_sources.t Memo.Lazy.t
+  ; source_dir : Source_tree.Dir.t option
   }
 
 and kind =
@@ -32,9 +33,10 @@ and kind =
   | Group_root of t list
   | Group_part
 
-let empty kind ~dir =
+let empty kind ~dir ~source_dir =
   { kind
   ; dir
+  ; source_dir
   ; text_files = Filename.Set.empty
   ; ml = Memo.Lazy.of_val Ml_sources.empty
   ; mlds = Memo.Lazy.of_val []
@@ -52,11 +54,11 @@ module Standalone_or_root = struct
 
   type nonrec t = { contents : standalone_or_root Memo.Lazy.t }
 
-  let empty ~dir =
+  let empty ~dir ~source_dir =
     { contents =
         Memo.Lazy.create (fun () ->
           Memo.return
-            { root = empty Standalone ~dir
+            { root = empty Standalone ~dir ~source_dir
             ; rules = Rules.empty
             ; subdirs = Path.Build.Map.empty
             })
@@ -84,6 +86,7 @@ type triage =
   | Group_part of Path.Build.t
 
 let dir t = t.dir
+let source_dir t = t.source_dir
 let coq t = Memo.Lazy.force t.coq
 let ocaml t = Memo.Lazy.force t.ml
 let artifacts t = Memo.Lazy.force t.ml >>= Ml_sources.artifacts
@@ -118,40 +121,6 @@ let mlds t ~(stanza : Documentation.t) =
       ]
 ;;
 
-let build_mlds_map stanzas ~dir ~files =
-  let mlds =
-    Memo.lazy_ (fun () ->
-      Filename.Set.fold files ~init:Filename.Map.empty ~f:(fun fn acc ->
-        (* TODO this doesn't handle [.foo.mld] correctly *)
-        match String.lsplit2 fn ~on:'.' with
-        | Some (s, "mld") -> Filename.Map.set acc s fn
-        | _ -> acc)
-      |> Memo.return)
-  in
-  Dune_file.find_stanzas stanzas Documentation.key
-  >>= Memo.parallel_map ~f:(fun (doc : Documentation.t) ->
-    let+ mlds =
-      let+ mlds = Memo.Lazy.force mlds in
-      Ordered_set_lang.Unordered_string.eval
-        doc.mld_files
-        ~standard:mlds
-        ~key:Fun.id
-        ~parse:(fun ~loc s ->
-          match Filename.Map.find mlds s with
-          | Some s -> s
-          | None ->
-            User_error.raise
-              ~loc
-              [ Pp.textf
-                  "%s.mld doesn't exist in %s"
-                  s
-                  (Path.to_string_maybe_quoted
-                     (Path.drop_optional_build_context (Path.build dir)))
-              ])
-    in
-    doc, List.map (Filename.Map.values mlds) ~f:(Path.Build.relative dir))
-;;
-
 module rec Load : sig
   val get : Super_context.t -> dir:Path.Build.t -> t Memo.t
   val triage : Super_context.t -> dir:Path.Build.t -> triage Memo.t
@@ -247,6 +216,12 @@ end = struct
       (Path.to_string_maybe_quoted (Path.build dir))
   ;;
 
+  let mlds ~sctx ~dir ~dune_file ~files =
+    Memo.lazy_ (fun () ->
+      let* expander = Super_context.expander sctx ~dir in
+      Doc_sources.build_mlds_map dune_file ~dir ~files expander)
+  ;;
+
   let make_standalone sctx st_dir ~dir (d : Dune_file.t) =
     let human_readable_description () = human_readable_description dir in
     { Standalone_or_root.contents =
@@ -264,7 +239,10 @@ end = struct
               let src_dir = Dune_file.dir d in
               stanzas >>= load_text_files sctx st_dir ~src_dir ~dir)
           in
-          let dirs = [ { Source_file_dir.dir; path_to_root = []; files } ] in
+          let dirs =
+            [ { Source_file_dir.dir; path_to_root = []; files; source_dir = Some st_dir }
+            ]
+          in
           let ml =
             Memo.lazy_ (fun () ->
               let lookup_vlib = lookup_vlib sctx ~current_dir:dir in
@@ -283,16 +261,18 @@ end = struct
                     ~lookup_vlib
                     ~dirs)
           in
+          let mlds = mlds ~sctx ~dir ~dune_file:d ~files in
           { Standalone_or_root.root =
               { kind = Standalone
+              ; source_dir = Some st_dir
               ; dir
               ; text_files = files
               ; ml
-              ; mlds = Memo.lazy_ (fun () -> build_mlds_map d ~dir ~files)
+              ; mlds
               ; foreign_sources =
                   Memo.lazy_ (fun () ->
                     let dune_version = Dune_project.dune_version project in
-                    stanzas >>| Foreign_sources.make ~dune_version ~dirs)
+                    stanzas >>= Foreign_sources.make ~dir ~dune_version ~dirs)
               ; coq =
                   Memo.lazy_ (fun () ->
                     stanzas >>| Coq_sources.of_dir ~dir ~include_subdirs ~dirs)
@@ -304,9 +284,9 @@ end = struct
   ;;
 
   let make_group_root
-    sctx
-    ~dir
-    { Dir_status.Group_root.qualification; dune_file; source_dir; components }
+        sctx
+        ~dir
+        { Dir_status.Group_root.qualification; dune_file; source_dir; components }
     =
     let include_subdirs =
       let loc, qualif_mode = qualification in
@@ -318,89 +298,105 @@ end = struct
       Memo.lazy_
         ~human_readable_description:(fun () -> human_readable_description dir)
         (fun () ->
-          let ctx = Super_context.context sctx in
-          let stanzas = Dune_file.stanzas dune_file in
-          let project = Dune_file.project dune_file in
-          let+ (files, subdirs), rules =
-            Rules.collect (fun () ->
-              Memo.fork_and_join
-                (fun () ->
-                  stanzas
-                  >>= load_text_files
-                        sctx
-                        source_dir
-                        ~src_dir:(Dune_file.dir dune_file)
-                        ~dir)
-                (fun () ->
-                  Memo.parallel_map
-                    components
-                    ~f:(fun { dir; path_to_group_root; source_dir; stanzas } ->
-                      let+ files =
-                        load_text_files
+           let ctx = Super_context.context sctx in
+           let stanzas = Dune_file.stanzas dune_file in
+           let project = Dune_file.project dune_file in
+           let+ (files, subdirs), rules =
+             Rules.collect (fun () ->
+               Memo.fork_and_join
+                 (fun () ->
+                    stanzas
+                    >>= load_text_files
                           sctx
                           source_dir
-                          stanzas
-                          ~src_dir:(Source_tree.Dir.path source_dir)
-                          ~dir
-                      in
-                      { Source_file_dir.dir; path_to_root = path_to_group_root; files })))
-          in
-          let dirs = { Source_file_dir.dir; path_to_root = []; files } :: subdirs in
-          let lib_config =
-            let+ ocaml = Context.ocaml ctx in
-            ocaml.lib_config
-          in
-          let ml =
-            Memo.lazy_ (fun () ->
-              let lookup_vlib = lookup_vlib sctx ~current_dir:dir in
-              let libs = Scope.DB.find_by_dir dir >>| Scope.libs in
-              let* expander = Super_context.expander sctx ~dir in
-              stanzas
-              >>= Ml_sources.make
-                    ~expander
-                    ~dir
-                    ~project
-                    ~libs
-                    ~lib_config
-                    ~loc
-                    ~lookup_vlib
-                    ~include_subdirs
-                    ~dirs)
-          in
-          let foreign_sources =
-            Memo.lazy_ (fun () ->
-              let dune_version = Dune_project.dune_version project in
-              stanzas >>| Foreign_sources.make ~dune_version ~dirs)
-          in
-          let coq =
-            Memo.lazy_ (fun () ->
-              stanzas >>| Coq_sources.of_dir ~dir ~dirs ~include_subdirs)
-          in
-          let subdirs =
-            List.map subdirs ~f:(fun { Source_file_dir.dir; path_to_root = _; files } ->
-              { kind = Group_part
-              ; dir
-              ; text_files = files
-              ; ml
-              ; foreign_sources
-              ; mlds = Memo.lazy_ (fun () -> build_mlds_map dune_file ~dir ~files)
-              ; coq
-              })
-          in
-          let root =
-            { kind = Group_root subdirs
-            ; dir
-            ; text_files = files
-            ; ml
-            ; foreign_sources
-            ; mlds = Memo.lazy_ (fun () -> build_mlds_map dune_file ~dir ~files)
-            ; coq
-            }
-          in
-          { Standalone_or_root.root
-          ; rules
-          ; subdirs = Path.Build.Map.of_list_map_exn subdirs ~f:(fun x -> x.dir, x)
-          })
+                          ~src_dir:(Dune_file.dir dune_file)
+                          ~dir)
+                 (fun () ->
+                    Memo.parallel_map
+                      components
+                      ~f:(fun { dir; path_to_group_root; source_dir; stanzas } ->
+                        let+ files =
+                          load_text_files
+                            sctx
+                            source_dir
+                            stanzas
+                            ~src_dir:(Source_tree.Dir.path source_dir)
+                            ~dir
+                        in
+                        { Source_file_dir.dir
+                        ; path_to_root = path_to_group_root
+                        ; files
+                        ; source_dir = Some source_dir
+                        })))
+           in
+           let dirs =
+             { Source_file_dir.dir
+             ; path_to_root = []
+             ; files
+             ; source_dir = Some source_dir
+             }
+             :: subdirs
+           in
+           let lib_config =
+             let+ ocaml = Context.ocaml ctx in
+             ocaml.lib_config
+           in
+           let ml =
+             Memo.lazy_ (fun () ->
+               let lookup_vlib = lookup_vlib sctx ~current_dir:dir in
+               let libs = Scope.DB.find_by_dir dir >>| Scope.libs in
+               let* expander = Super_context.expander sctx ~dir in
+               stanzas
+               >>= Ml_sources.make
+                     ~expander
+                     ~dir
+                     ~project
+                     ~libs
+                     ~lib_config
+                     ~loc
+                     ~lookup_vlib
+                     ~include_subdirs
+                     ~dirs)
+           in
+           let foreign_sources =
+             Memo.lazy_ (fun () ->
+               let dune_version = Dune_project.dune_version project in
+               stanzas >>= Foreign_sources.make ~dir ~dune_version ~dirs)
+           in
+           let coq =
+             Memo.lazy_ (fun () ->
+               stanzas >>| Coq_sources.of_dir ~dir ~dirs ~include_subdirs)
+           in
+           let mlds = mlds ~sctx ~dir ~dune_file ~files in
+           let subdirs =
+             List.map
+               subdirs
+               ~f:(fun { Source_file_dir.dir; path_to_root = _; files; source_dir } ->
+                 { kind = Group_part
+                 ; source_dir
+                 ; dir
+                 ; text_files = files
+                 ; ml
+                 ; foreign_sources
+                 ; mlds
+                 ; coq
+                 })
+           in
+           let root =
+             { kind = Group_root subdirs
+             ; source_dir = Some source_dir
+             ; dir
+             ; text_files = files
+             ; ml
+             ; foreign_sources
+             ; mlds
+             ; coq
+             }
+           in
+           { Standalone_or_root.root
+           ; rules
+           ; subdirs = Path.Build.Map.of_list_map_exn subdirs ~f:(fun x -> x.dir, x)
+           })
     in
     { Standalone_or_root.contents }
   ;;
@@ -410,8 +406,11 @@ end = struct
     >>= function
     | Is_component_of_a_group_but_not_the_root { group_root; stanzas = _ } ->
       Memo.return @@ Group_part group_root
-    | Lock_dir | Generated | Source_only _ ->
-      Memo.return @@ Standalone_or_root (Standalone_or_root.empty ~dir)
+    | Generated ->
+      Memo.return @@ Standalone_or_root (Standalone_or_root.empty ~dir ~source_dir:None)
+    | Lock_dir source_dir | Source_only source_dir ->
+      Memo.return
+      @@ Standalone_or_root (Standalone_or_root.empty ~dir ~source_dir:(Some source_dir))
     | Standalone (st_dir, d) ->
       Memo.return @@ Standalone_or_root (make_standalone sctx st_dir ~dir d)
     | Group_root root ->
