@@ -19,9 +19,6 @@ end
 module Lib : sig
   type t
 
-  val main_module_name : t -> Module_name.t option
-  val dir_of_name : Lib_name.t -> Path.Local.t
-  val wrapped : t -> Wrapped.t option
   val info : t -> Path.t Lib_info.t
   val external_location : t -> External_location.t option
   val of_findlib : Path.t Lib_info.t -> External_location.t -> t
@@ -48,7 +45,7 @@ module Entry : sig
 
         - are not built with dune
 
-        - have a [META] file with an unsatisfied [exist_if] field
+        - have a [META] file with an unsatisfied [exists_if] field
 
         Dune itself never produces hidden libraries. *)
 
@@ -70,14 +67,36 @@ type t =
   ; files : (Section.t * path list) list
   }
 
+type replace_info
+
+(** Replace sections by the external paths for sites to refer to them.
+    It saves some information that needs to be passed to [Or_meta.pp] using
+    [~encoding:Absolute info]. *)
+val replace_site_sections
+  :  t
+  -> get_location:(Section.t -> Package.Name.t -> Path.t)
+  -> t * replace_info
+
 val to_dyn : t Dyn.builder
 
+type encoding =
+  | Absolute of replace_info
+  | Relative
+
 module Or_meta : sig
   type nonrec t =
     | Use_meta
     | Dune_package of t
 
-  val pp : dune_version:Dune_lang.Syntax.Version.t -> Format.formatter -> t -> unit
+  val pp
+    :  encoding:encoding
+    -> dune_version:Dune_lang.Syntax.Version.t
+    -> Format.formatter
+    -> t
+    -> unit
+
+  val pp_use_meta : dune_version:Dune_lang.Syntax.Version.t -> Format.formatter -> unit
+  val parse : Path.t -> Lexbuf.t -> (t, User_message.t) result
   val load : Path.t -> (t, User_message.t) result Memo.t
   val to_dyn : t Dyn.builder
 end
