@@ -7,15 +7,13 @@ type t
 val make
   :  dir:Path.Build.t
   -> inherit_from:t Memo.Lazy.t option
-  -> scope:Scope.t
   -> config_stanza:Dune_env.t
   -> profile:Profile.t
-  -> expander_for_artifacts:Expander.t Memo.Lazy.t
-  -> default_env:Env.t
-  -> default_artifacts:Artifacts.t
+  -> expander:Expander.t Memo.t
+  -> default_env:Env.t Memo.t
+  -> default_artifacts:Artifacts.t Memo.t
   -> t
 
-val scope : t -> Scope.t
 val external_env : t -> Env.t Memo.t
 
 (** Binaries that are symlinked in the associated .bin directory of [dir]. This
