@@ -5,23 +5,31 @@ open Import
 type t
 
 val dir : t -> Path.Build.t
-val context : t -> Context.t
+val context : t -> Context_name.t
+val project : t -> Dune_project.t
 
 val make_root
-  :  scope:Scope.t
-  -> scope_host:Scope.t
+  :  project:Dune_project.t
+  -> scope:Scope.t Memo.t
+  -> scope_host:Scope.t Memo.t
   -> context:Context.t
-  -> env:Env.t
-  -> lib_artifacts:Lib.DB.t
-  -> lib_artifacts_host:Lib.DB.t
-  -> artifacts_host:Artifacts.t
+  -> env:Env.t Memo.t
+  -> public_libs:Lib.DB.t Memo.t
+  -> public_libs_host:Lib.DB.t Memo.t
+  -> artifacts_host:Artifacts.t Memo.t
   -> t
 
 val set_local_env_var : t -> var:string -> value:string Action_builder.t -> t
-val set_dir : t -> dir:Path.Build.t -> t
-val set_scope : t -> scope:Scope.t -> scope_host:Scope.t -> t
-val set_artifacts : t -> artifacts_host:Artifacts.t -> t
-val set_lookup_ml_sources : t -> f:(dir:Path.Build.t -> Artifacts_obj.t Memo.t) -> t
+
+val set_scope
+  :  t
+  -> dir:Path.Build.t
+  -> project:Dune_project.t
+  -> scope:Scope.t Memo.t
+  -> scope_host:Scope.t Memo.t
+  -> t
+
+val set_artifacts : t -> artifacts_host:Artifacts.t Memo.t -> t
 
 module Expanding_what : sig
   type t =
@@ -57,7 +65,7 @@ end
 type value = Value.t list Deps.t
 
 val add_bindings_full : t -> bindings:value Pform.Map.t -> t
-val extend_env : t -> env:Env.t -> t
+val extend_env : t -> env:Env.t Memo.t -> t
 
 val expand
   :  t
@@ -68,6 +76,7 @@ val expand
 val expand_path : t -> String_with_vars.t -> Path.t Action_builder.t
 val expand_str : t -> String_with_vars.t -> string Action_builder.t
 val expand_pform : t -> Value.t list Action_builder.t String_with_vars.expander
+val expand_str_partial : t -> String_with_vars.t -> String_with_vars.t Action_builder.t
 
 module No_deps : sig
   (** Same as [expand_xxx] but disallow percent forms that introduce action
@@ -91,32 +100,9 @@ module With_deps_if_necessary : sig
   val expand_path : t -> String_with_vars.t -> Path.t list Deps.t
 end
 
-module With_reduced_var_set : sig
-  val expand
-    :  context:Context.t
-    -> dir:Path.Build.t
-    -> String_with_vars.t
-    -> Value.t Memo.t
-
-  val expand_str
-    :  context:Context.t
-    -> dir:Path.Build.t
-    -> String_with_vars.t
-    -> string Memo.t
-
-  val expand_str_partial
-    :  context:Context.t
-    -> dir:Path.Build.t
-    -> String_with_vars.t
-    -> String_with_vars.t Memo.t
-
-  val eval_blang : context:Context.t -> dir:Path.Build.t -> Blang.t -> bool Memo.t
-end
-
 val expand_ordered_set_lang
-  :  Ordered_set_lang.Unexpanded.t
-  -> dir:Path.t
-  -> f:Value.t list Action_builder.t String_with_vars.expander
+  :  t
+  -> Ordered_set_lang.Unexpanded.t
   -> Ordered_set_lang.t Action_builder.t
 
 (** Expand forms of the form (:standard \ foo bar). Expansion is only possible
@@ -130,14 +116,12 @@ val expand_and_eval_set
 
 val eval_blang : t -> Blang.t -> bool Memo.t
 val map_exe : t -> Path.t -> Path.t
-val artifacts : t -> Artifacts.t
-
-val expand_locks
-  :  base:[ `Of_expander | `This of Path.t ]
-  -> t
-  -> Locks.t
-  -> Path.t list Action_builder.t
+val artifacts : t -> Artifacts.t Memo.t
+val expand_locks : t -> Locks.t -> Path.t list Action_builder.t
 
 val foreign_flags
   : (dir:Path.Build.t -> string list Action_builder.t Foreign_language.Dict.t Memo.t)
       Fdecl.t
+
+val lookup_artifacts : (dir:Path.Build.t -> Artifacts_obj.t Memo.t) Fdecl.t
+val to_expander0 : t -> Expander0.t
