@@ -8,7 +8,7 @@ type t
 module Unavailable_reason : sig
   type t =
     | Not_found
-    (** The package is hidden because it contains an unsatisfied 'exist_if'
+    (** The package is hidden because it contains an unsatisfied 'exists_if'
         clause *)
     | Invalid_dune_package of User_message.t
 
@@ -32,6 +32,10 @@ val all_broken_packages : t -> (Package.Name.t * User_message.t) list Memo.t
 
 val create : Context_name.t -> t Memo.t
 
+(** Create a findlib database with a given list of paths corresponding to the
+    "path" field in the findlib.conf file. *)
+val create_with_paths : paths:Path.t list -> Context_name.t -> t Memo.t
+
 module For_tests : sig
   val create : paths:Path.t list -> lib_config:Lib_config.t -> t Memo.t
 end
