@@ -1,4 +1,5 @@
 open Import
+module Ocaml_flags = Dune_lang.Ocaml_flags
 
 module Backend = struct
   let name = Sub_system_name.of_string "inline_tests.backend"
@@ -183,3 +184,9 @@ module Tests = struct
      purposes *)
   let encode _t = assert false
 end
+
+let inline_test_dirname lib_name =
+  sprintf ".%s.inline-tests" (Lib_name.Local.to_string lib_name)
+;;
+
+let inline_test_runner = "inline-test-runner"
