@@ -20,11 +20,13 @@ let compute_env ~mode =
             let+ v = Expander.eval_blang expander enabled_if in
             Some (Blang.Const v)
         in
+        let runtest_alias = Option.first_some local.runtest_alias parent.runtest_alias in
+        Option.iter ~f:Alias0.register_as_standard runtest_alias;
         { Js_of_ocaml.Env.compilation_mode =
             Option.first_some local.compilation_mode parent.compilation_mode
         ; sourcemap = Option.first_some local.sourcemap parent.sourcemap
         ; enabled_if
-        ; runtest_alias = Option.first_some local.runtest_alias parent.runtest_alias
+        ; runtest_alias
         ; flags =
             Js_of_ocaml.Flags.make
               ~spec:local.flags
@@ -41,95 +43,6 @@ let js_env = compute_env ~mode:JS
 let wasm_env = compute_env ~mode:Wasm
 let jsoo_env ~dir ~mode = (Js_of_ocaml.Mode.select ~mode ~js:js_env ~wasm:wasm_env) ~dir
 
-module Config : sig
-  type t
-
-  val all : t list
-  val path : t -> string
-  val of_string : string -> t
-  val of_flags : string list -> t
-  val to_flags : t -> string list
-end = struct
-  type t =
-    { js_string : bool option
-    ; effects : bool option
-    ; toplevel : bool option
-    }
-
-  let default = { js_string = None; effects = None; toplevel = None }
-  let bool_opt = [ None; Some true; Some false ]
-
-  let all =
-    List.concat_map bool_opt ~f:(fun js_string ->
-      List.concat_map bool_opt ~f:(fun effects ->
-        List.concat_map bool_opt ~f:(fun toplevel -> [ { js_string; effects; toplevel } ])))
-  ;;
-
-  let get t =
-    List.filter_map
-      [ "use-js-string", t.js_string; "effects", t.effects; "toplevel", t.toplevel ]
-      ~f:(fun (n, v) ->
-        match v with
-        | None -> None
-        | Some v -> Some (n, v))
-  ;;
-
-  let set acc name v =
-    match name with
-    | "use-js-string" -> { acc with js_string = Some v }
-    | "effects" -> { acc with effects = Some v }
-    | "toplevel" -> { acc with toplevel = Some v }
-    | _ -> acc
-  ;;
-
-  let path t =
-    if t = default
-    then "default"
-    else
-      List.map (get t) ~f:(function
-        | x, true -> x
-        | x, false -> "!" ^ x)
-      |> String.concat ~sep:"+"
-  ;;
-
-  let of_string x =
-    match x with
-    | "default" -> default
-    | _ ->
-      List.fold_left (String.split ~on:'+' x) ~init:default ~f:(fun acc name ->
-        match String.drop_prefix ~prefix:"!" name with
-        | Some name -> set acc name false
-        | None -> set acc name true)
-  ;;
-
-  let of_flags l =
-    let rec loop acc = function
-      | [] -> acc
-      | "--enable" :: name :: rest -> loop (set acc name true) rest
-      | maybe_enable :: rest when String.is_prefix maybe_enable ~prefix:"--enable=" ->
-        (match String.drop_prefix maybe_enable ~prefix:"--enable=" with
-         | Some name -> loop (set acc name true) rest
-         | _ -> assert false)
-      | "--disable" :: name :: rest -> loop (set acc name false) rest
-      | maybe_disable :: rest when String.is_prefix maybe_disable ~prefix:"--disable=" ->
-        (match String.drop_prefix maybe_disable ~prefix:"--disable=" with
-         | Some name -> loop (set acc name false) rest
-         | _ -> assert false)
-      | "--toplevel" :: rest -> loop (set acc "toplevel" true) rest
-      | _ :: rest -> loop acc rest
-    in
-    loop default l
-  ;;
-
-  let to_flags t =
-    List.concat_map (get t) ~f:(function
-      | "toplevel", true -> [ "--toplevel" ]
-      | "toplevel", false -> []
-      | name, true -> [ "--enable"; name ]
-      | name, false -> [ "--disable"; name ])
-  ;;
-end
-
 module Version = struct
   type t = int * int
 
@@ -174,7 +87,193 @@ module Version = struct
   ;;
 end
 
+module Config : sig
+  type t
+
+  val all : t list
+  val path : t -> string
+  val of_string : string -> t
+  val of_flags : string list -> t
+  val to_flags : jsoo_version:Version.t option -> t -> string list
+  val remove_config_flags : string list -> string list
+end = struct
+  type effects_backend =
+    | Cps
+    | Double_translation
+
+  type t =
+    { js_string : bool option
+    ; effects : effects_backend option
+    ; toplevel : bool option
+    }
+
+  let default = { js_string = None; effects = None; toplevel = None }
+  let bool_opt = [ None; Some true; Some false ]
+  let effects_opt = [ None; Some Cps; Some Double_translation ]
+
+  let all =
+    List.concat_map bool_opt ~f:(fun js_string ->
+      List.concat_map effects_opt ~f:(fun effects ->
+        List.concat_map bool_opt ~f:(fun toplevel -> [ { js_string; effects; toplevel } ])))
+  ;;
+
+  let enable name acc =
+    match name with
+    | "use-js-string" -> { acc with js_string = Some true }
+    | "effects" ->
+      (* [--enable effects], used alone, implies [--effects=cps] *)
+      (match acc.effects with
+       | None -> { acc with effects = Some Cps }
+       | Some _ -> acc)
+    | "toplevel" -> { acc with toplevel = Some true }
+    | _ -> acc
+  ;;
+
+  let disable name acc =
+    match name with
+    | "use-js-string" -> { acc with js_string = Some false }
+    | "effects" -> { acc with effects = None }
+    | "toplevel" -> { acc with toplevel = Some false }
+    | _ -> acc
+  ;;
+
+  let string_of_effects = function
+    | Cps -> "cps"
+    | Double_translation -> "double-translation"
+  ;;
+
+  let path t =
+    if t = default
+    then "default"
+    else (
+      let of_bool_opt key =
+        Option.map ~f:(function
+          | true -> key
+          | false -> "!" ^ key)
+      in
+      List.filter_opt
+        [ of_bool_opt "use-js-string" t.js_string
+        ; Option.map t.effects ~f:(fun e -> "effects=" ^ string_of_effects e)
+        ; of_bool_opt "toplevel" t.toplevel
+        ]
+      |> String.concat ~sep:"+")
+  ;;
+
+  let effects_of_string = function
+    | "cps" -> Some Cps
+    | "double-translation" -> Some Double_translation
+    | _ -> None
+  ;;
+
+  let of_string x =
+    match x with
+    | "default" -> default
+    | _ ->
+      List.fold_left (String.split ~on:'+' x) ~init:default ~f:(fun acc name ->
+        match
+          String.drop_prefix ~prefix:"!" name, String.drop_prefix ~prefix:"effects=" name
+        with
+        | Some name, _ -> disable name acc
+        | None, None -> enable name acc
+        | None, Some backend ->
+          (match effects_of_string backend with
+           | Some backend -> { acc with effects = Some backend }
+           | None -> acc))
+  ;;
+
+  let of_flags l =
+    let rec loop acc = function
+      | [] -> acc
+      | "--enable" :: name :: rest -> loop (enable name acc) rest
+      | maybe_enable :: rest when String.is_prefix maybe_enable ~prefix:"--enable=" ->
+        (match String.drop_prefix maybe_enable ~prefix:"--enable=" with
+         | Some name -> loop (enable name acc) rest
+         | _ -> assert false)
+      | "--disable" :: name :: rest -> loop (disable name acc) rest
+      | maybe_disable :: rest when String.is_prefix maybe_disable ~prefix:"--disable=" ->
+        (match String.drop_prefix maybe_disable ~prefix:"--disable=" with
+         | Some name -> loop (disable name acc) rest
+         | _ -> assert false)
+      | "--toplevel" :: rest -> loop (enable "toplevel" acc) rest
+      | "--effects" :: "cps" :: rest -> loop { acc with effects = Some Cps } rest
+      | "--effects" :: "double-translation" :: rest ->
+        loop { acc with effects = Some Double_translation } rest
+      | maybe_effects :: rest when String.is_prefix maybe_effects ~prefix:"--effects=" ->
+        let backend =
+          Option.bind
+            (String.drop_prefix maybe_effects ~prefix:"--effects=")
+            ~f:effects_of_string
+        in
+        (match backend with
+         | Some backend -> loop { acc with effects = Some backend } rest
+         | None -> loop acc rest)
+      | _ :: rest -> loop acc rest
+    in
+    loop default l
+  ;;
+
+  let backward_compatible_effects ~jsoo_version str =
+    match str with
+    | None ->
+      (* For jsoo, this means unsupported effects. For wasmoo, this means effects go
+         through the Javascript Promise API. *)
+      None
+    | Some Cps ->
+      let v6_or_higher =
+        match jsoo_version with
+        | Some v ->
+          (match Version.compare v (6, 0) with
+           | Gt | Eq -> true
+           | Lt -> false)
+        | None -> false
+      in
+      if v6_or_higher then Some "--effects=cps" else Some "--enable=effects"
+    | Some Double_translation ->
+      (* For js_of_ocaml < 6.0, this flag does not exist and will raise an error,
+         which is fine. *)
+      Some "--effects=double-translation"
+  ;;
+
+  let to_flags ~jsoo_version t =
+    List.filter_opt
+      [ (match t.toplevel with
+         | Some true -> Some "--toplevel"
+         | _ -> None)
+      ; backward_compatible_effects ~jsoo_version t.effects
+      ; (match t.js_string with
+         | Some true -> Some "--enable=use-js-string"
+         | Some false -> Some "--disable=use-js-string"
+         | None -> None)
+      ]
+  ;;
+
+  let remove_config_flags flags =
+    let rec loop acc = function
+      | [] -> acc
+      | "--enable" :: ("effects" | "use-js-string") :: rest -> loop acc rest
+      | maybe_enable :: rest when String.is_prefix maybe_enable ~prefix:"--enable=" ->
+        (match String.drop_prefix maybe_enable ~prefix:"--enable=" with
+         | Some ("effects" | "use-js-string") -> loop acc rest
+         | Some _ -> loop (maybe_enable :: acc) rest
+         | None -> assert false)
+      | "--disable" :: ("effects" | "use-js-string") :: rest -> loop acc rest
+      | maybe_disable :: rest when String.is_prefix maybe_disable ~prefix:"--disable=" ->
+        (match String.drop_prefix maybe_disable ~prefix:"--disable=" with
+         | Some ("effects" | "use-js-string") -> loop acc rest
+         | Some _ -> loop (maybe_disable :: acc) rest
+         | None -> assert false)
+      | "--effects" :: _backend :: rest -> loop acc rest
+      | maybe_effects :: rest when String.is_prefix maybe_effects ~prefix:"--effects=" ->
+        loop acc rest
+      | "--toplevel" :: rest -> loop acc rest
+      | other :: rest -> loop (other :: acc) rest
+    in
+    loop [] flags |> List.rev
+  ;;
+end
+
 let install_jsoo_hint = "opam install js_of_ocaml-compiler"
+let install_wasmoo_hint = "opam install wasm_of_ocaml-compiler"
 
 let in_build_dir (ctx : Build_context.t) ~config args =
   Path.Build.L.relative ctx.build_dir (".js" :: Config.path config :: args)
@@ -209,8 +308,21 @@ let jsoo ~dir sctx =
 ;;
 
 let wasmoo ~dir sctx =
-  (* TODO add a hint when wasm_of_ocaml released on opam *)
-  Super_context.resolve_program sctx ~dir ~loc:None "wasm_of_ocaml"
+  Super_context.resolve_program
+    sctx
+    ~dir
+    ~loc:None
+    ~hint:install_wasmoo_hint
+    "wasm_of_ocaml"
+;;
+
+let jsoo_has_shapes jsoo_version =
+  match jsoo_version with
+  | Some version ->
+    (match Version.compare version (6, 1) with
+     | Lt -> false
+     | Gt | Eq -> true)
+  | None -> false
 ;;
 
 type sub_command =
@@ -230,16 +342,16 @@ let js_of_ocaml_flags t ~dir ~mode (spec : Js_of_ocaml.Flags.Spec.t) =
 ;;
 
 let js_of_ocaml_rule
-  sctx
-  ~(mode : Js_of_ocaml.Mode.t)
-  ~sub_command
-  ~dir
-  ~(flags : _ Js_of_ocaml.Flags.t)
-  ~config
-  ~spec
-  ~target
-  ~sourcemap
-  ~directory_targets
+      sctx
+      ~(mode : Js_of_ocaml.Mode.t)
+      ~sub_command
+      ~dir
+      ~(flags : _ Js_of_ocaml.Flags.t)
+      ~config
+      ~spec
+      ~target
+      ~sourcemap
+      ~directory_targets
   =
   let open Action_builder.O in
   let jsoo =
@@ -254,6 +366,13 @@ let js_of_ocaml_rule
     | Link -> flags.link
     | Build_runtime -> flags.build_runtime
   in
+  let flags =
+    (* Avoid duplicating flags that are covered by the config *)
+    Action_builder.map flags ~f:(fun flags ->
+      match config with
+      | None -> flags
+      | Some _ -> Config.remove_config_flags flags)
+  in
   Command.run_dyn_prog
     ~dir:(Path.build dir)
     jsoo
@@ -275,9 +394,14 @@ let js_of_ocaml_rule
        | None -> S []
        | Some config ->
          Dyn
-           (Action_builder.map config ~f:(fun config ->
-              Command.Args.S
-                (List.map (Config.to_flags config) ~f:(fun x -> Command.Args.A x)))))
+           (let+ config = config
+            and+ jsoo_version =
+              let* jsoo = jsoo in
+              Action_builder.of_memo (Version.jsoo_version jsoo)
+            in
+            Command.Args.S
+              (List.map (Config.to_flags ~jsoo_version config) ~f:(fun x ->
+                 Command.Args.A x))))
     ; A "-o"
     ; Target target
     ; spec
@@ -326,15 +450,15 @@ let standalone_runtime_rule ~mode cc ~runtime_files ~target ~flags ~sourcemap =
 ;;
 
 let exe_rule
-  ~mode
-  cc
-  ~linkall
-  ~runtime_files
-  ~src
-  ~target
-  ~directory_targets
-  ~flags
-  ~sourcemap
+      ~mode
+      cc
+      ~linkall
+      ~runtime_files
+      ~src
+      ~target
+      ~directory_targets
+      ~flags
+      ~sourcemap
   =
   let dir = Compilation_context.dir cc in
   let sctx = Compilation_context.super_context cc in
@@ -407,26 +531,27 @@ let jsoo_archives ~mode ctx config lib =
            ]))
 ;;
 
+let cmo_js_of_module ~mode m =
+  Module_name.Unique.artifact_filename
+    (Module.obj_name m)
+    ~ext:(Js_of_ocaml.Ext.cmo ~mode)
+;;
+
 let link_rule
-  ~mode
-  cc
-  ~runtime
-  ~target
-  ~directory_targets
-  ~obj_dir
-  cm
-  ~flags
-  ~linkall
-  ~link_time_code_gen
-  ~sourcemap
+      ~mode
+      cc
+      ~runtime
+      ~target
+      ~directory_targets
+      ~obj_dir
+      cm
+      ~flags
+      ~linkall
+      ~link_time_code_gen
+      ~sourcemap
   =
   let sctx = Compilation_context.super_context cc in
   let dir = Compilation_context.dir cc in
-  let mod_name m =
-    Module_name.Unique.artifact_filename
-      (Module.obj_name m)
-      ~ext:(Js_of_ocaml.Ext.cmo ~mode)
-  in
   let ctx = Super_context.context sctx |> Context.build_context in
   let get_all =
     let open Action_builder.O in
@@ -452,12 +577,13 @@ let link_rule
     let special_units =
       List.concat_map to_link ~f:(function
         | Lib_flags.Lib_and_module.Lib _lib -> []
-        | Module (obj_dir, m) -> [ in_obj_dir' ~obj_dir ~config:None [ mod_name m ] ])
+        | Module (obj_dir, m) ->
+          [ in_obj_dir' ~obj_dir ~config:None [ cmo_js_of_module ~mode m ] ])
     in
     let all_libs = List.concat_map libs ~f:(jsoo_archives ~mode ctx config) in
     let all_other_modules =
       List.map cm ~f:(fun m ->
-        Path.build (in_obj_dir ~obj_dir ~config:None [ mod_name m ]))
+        Path.build (in_obj_dir ~obj_dir ~config:None [ cmo_js_of_module ~mode m ]))
     in
     let std_exit =
       Path.build
@@ -491,8 +617,25 @@ let link_rule
     ~sourcemap
 ;;
 
-let build_cm' sctx ~dir ~in_context ~mode ~src ~target ~config ~sourcemap =
-  let spec = Command.Args.Dep src in
+let build_cm' sctx ~dir ~in_context ~mode ~src ~target ~config ~shapes ~sourcemap =
+  let spec =
+    Command.Args.(
+      S
+        [ Dep src
+        ; Dyn
+            (let open Action_builder.O in
+             let* jsoo_version =
+               let* jsoo = jsoo ~dir sctx in
+               Action_builder.of_memo @@ Version.jsoo_version jsoo
+             in
+             let+ shapes =
+               match jsoo_has_shapes jsoo_version with
+               | false -> Action_builder.return []
+               | true -> shapes
+             in
+             S (List.map shapes ~f:(fun s -> S [ A "--load-shape"; Dep s ])))
+        ])
+  in
   let flags = in_context.Js_of_ocaml.In_context.flags in
   js_of_ocaml_rule
     sctx
@@ -507,9 +650,30 @@ let build_cm' sctx ~dir ~in_context ~mode ~src ~target ~config ~sourcemap =
     ~sourcemap
 ;;
 
-let build_cm sctx ~dir ~in_context ~mode ~src ~obj_dir ~config =
+let build_cm cctx ~dir ~in_context ~mode ~src ~obj_dir ~deps ~config:config_opt =
   let name = with_js_ext ~mode (Path.basename src) in
-  let target = in_obj_dir ~obj_dir ~config [ name ] in
+  let target = in_obj_dir ~obj_dir ~config:config_opt [ name ] in
+  let sctx = Compilation_context.super_context cctx in
+  let ctx = Super_context.context sctx |> Context.build_context in
+  let shapes =
+    let open Action_builder.O in
+    let+ libs = Resolve.Memo.read (Compilation_context.requires_link cctx)
+    and+ deps = deps
+    and+ config =
+      match config_opt with
+      | None ->
+        let flags = in_context.Js_of_ocaml.In_context.flags in
+        js_of_ocaml_flags sctx ~dir ~mode flags
+        |> Action_builder.bind ~f:(fun (x : _ Js_of_ocaml.Flags.t) -> x.compile)
+        |> Action_builder.map ~f:Config.of_flags
+      | Some config -> Action_builder.return config
+    in
+    let deps = List.filter deps ~f:(fun m -> Module.has m ~ml_kind:Impl) in
+    (Path.build (in_build_dir ctx ~config [ "stdlib"; with_js_ext ~mode "stdlib.cma" ])
+     :: List.concat_map libs ~f:(fun lib -> jsoo_archives ~mode ctx config lib))
+    @ List.map deps ~f:(fun m ->
+      Path.build (in_obj_dir ~obj_dir ~config:config_opt [ cmo_js_of_module ~mode m ]))
+  in
   build_cm'
     sctx
     ~dir
@@ -517,7 +681,8 @@ let build_cm sctx ~dir ~in_context ~mode ~src ~obj_dir ~config =
     ~mode
     ~src
     ~target
-    ~config:(Option.map config ~f:Action_builder.return)
+    ~shapes
+    ~config:(Option.map config_opt ~f:Action_builder.return)
     ~sourcemap:Js_of_ocaml.Sourcemap.Inline
 ;;
 
@@ -534,6 +699,11 @@ let setup_separate_compilation_rules sctx components =
      | None -> Memo.return ()
      | Some pkg ->
        let info = Lib.info pkg in
+       let requires =
+         let open Resolve.Memo.O in
+         let* reqs = Lib.requires pkg in
+         Lib.closure ~linking:false reqs
+       in
        let lib_name = Lib_name.to_string (Lib.name pkg) in
        let* archives =
          let archives = (Lib_info.archives info).byte in
@@ -564,6 +734,23 @@ let setup_separate_compilation_rules sctx components =
            let target =
              in_build_dir build_context ~config [ lib_name; with_js_ext ~mode name ]
            in
+           let shapes =
+             let open Action_builder.O in
+             let+ requires = Resolve.Memo.read requires in
+             let l =
+               List.concat_map requires ~f:(fun lib ->
+                 jsoo_archives ~mode build_context config lib)
+             in
+             match lib_name with
+             | "stdlib" -> l
+             | _ ->
+               Path.build
+                 (in_build_dir
+                    build_context
+                    ~config
+                    [ "stdlib"; with_js_ext ~mode "stdlib.cma" ])
+               :: l
+           in
            build_cm'
              sctx
              ~dir
@@ -573,6 +760,7 @@ let setup_separate_compilation_rules sctx components =
              ~target
              ~config:(Some (Action_builder.return config))
              ~sourcemap:Js_of_ocaml.Sourcemap.Inline
+             ~shapes
            |> Super_context.add_rule sctx ~dir)))
 ;;
 
@@ -597,10 +785,10 @@ let js_of_ocaml_sourcemap t ~dir ~mode =
 ;;
 
 let jsoo_enabled
-  ~eval
-  ~dir
-  ~(in_context : Js_of_ocaml.In_context.t Js_of_ocaml.Mode.Pair.t)
-  ~mode
+      ~eval
+      ~dir
+      ~(in_context : Js_of_ocaml.In_context.t Js_of_ocaml.Mode.Pair.t)
+      ~mode
   =
   match (Js_of_ocaml.Mode.Pair.select ~mode in_context).enabled_if with
   | Some enabled_if -> eval enabled_if
@@ -619,10 +807,10 @@ let jsoo_enabled_modes ~expander ~dir ~in_context =
 ;;
 
 let jsoo_compilation_mode
-  t
-  ~dir
-  ~(in_context : Js_of_ocaml.In_context.t Js_of_ocaml.Mode.Pair.t)
-  ~mode
+      t
+      ~dir
+      ~(in_context : Js_of_ocaml.In_context.t Js_of_ocaml.Mode.Pair.t)
+      ~mode
   =
   match (Js_of_ocaml.Mode.Pair.select ~mode in_context).compilation_mode with
   | None -> js_of_ocaml_compilation_mode t ~dir ~mode
@@ -641,16 +829,16 @@ let jsoo_is_whole_program t ~dir ~in_context =
 ;;
 
 let build_exe
-  cc
-  ~loc
-  ~in_context
-  ~src
-  ~(obj_dir : Path.Build.t Obj_dir.t)
-  ~(top_sorted_modules : Module.t list Action_builder.t)
-  ~promote
-  ~linkall
-  ~link_time_code_gen
-  ~jsoo_mode:mode
+      cc
+      ~loc
+      ~in_context
+      ~src
+      ~(obj_dir : Path.Build.t Obj_dir.t)
+      ~(top_sorted_modules : Module.t list Action_builder.t)
+      ~promote
+      ~linkall
+      ~link_time_code_gen
+      ~jsoo_mode:mode
   =
   let sctx = Compilation_context.super_context cc in
   let dir = Compilation_context.dir cc in
