@@ -23,13 +23,38 @@ module L : sig
   type nonrec t = Lib.t list
 
   val to_iflags : Path.Set.t -> _ Command.Args.t
-  val include_paths : ?project:Dune_project.t -> t -> Lib_mode.t -> Path.Set.t
-  val include_flags : ?project:Dune_project.t -> t -> Lib_mode.t -> _ Command.Args.t
+
+  val include_paths
+    :  ?project:Dune_project.t
+    -> t
+    -> Lib_mode.t
+    -> Lib_config.t
+    -> Path.Set.t
+
+  val include_flags
+    :  ?project:Dune_project.t
+    -> direct_libs:t
+    -> hidden_libs:t
+    -> Lib_mode.t
+    -> Lib_config.t
+    -> _ Command.Args.t
+
+  val melange_emission_include_flags
+    :  ?project:Dune_project.t
+    -> t
+    -> Lib_config.t
+    -> _ Command.Args.t
+
   val c_include_flags : t -> Super_context.t -> _ Command.Args.t
-  val toplevel_ld_paths : t -> Path.Set.t
-  val toplevel_include_paths : t -> Path.Set.t
+  val toplevel_ld_paths : t -> Lib_config.t -> Path.Set.t
+  val toplevel_include_paths : t -> Lib_config.t -> Path.Set.t
 end
 
 (** The list of files that will be read by the compiler when linking an
     executable against this library *)
-val link_deps : Super_context.t -> Lib.t -> Link_mode.t -> Path.t list Memo.t
+val link_deps
+  :  Super_context.t
+  -> Lib.t
+  -> Link_mode.t
+  -> Lib_config.t
+  -> Path.t list Memo.t
