@@ -1,7 +1,7 @@
 open Import
 
 val foreign_rules
-  :  Foreign.Library.t
+  :  Foreign_library.t
   -> sctx:Super_context.t
   -> expander:Expander.t
   -> dir:Path.Build.t
@@ -12,7 +12,6 @@ val rules
   :  Library.t
   -> sctx:Super_context.t
   -> dir_contents:Dir_contents.t
-  -> dir:Path.Build.t
   -> expander:Expander.t
   -> scope:Scope.t
   -> (Compilation_context.t * Merlin.t) Memo.t
