@@ -11,9 +11,10 @@ let mode : t -> Mode.t = function
   | Byte_for_jsoo -> Byte
   | Native -> Native
   | Byte_with_stubs_statically_linked_in -> Byte
+;;
 
 let equal x y =
-  match (x, y) with
+  match x, y with
   | Byte, Byte -> true
   | Byte, _ -> false
   | _, Byte -> false
@@ -23,5 +24,5 @@ let equal x y =
   | Native, Native -> true
   | Native, _ -> false
   | _, Native -> false
-  | Byte_with_stubs_statically_linked_in, Byte_with_stubs_statically_linked_in
-    -> true
+  | Byte_with_stubs_statically_linked_in, Byte_with_stubs_statically_linked_in -> true
+;;
