@@ -14,11 +14,11 @@ module Emit = struct
     ; preprocess : Preprocess.With_instrumentation.t Preprocess.Per_module.t
     ; runtime_deps : Loc.t * Dep_conf.t list
     ; preprocessor_deps : Dep_conf.t list
+    ; lint : Preprocess.Without_instrumentation.t Preprocess.Per_module.t
     ; promote : Rule.Promote.t option
     ; compile_flags : Ordered_set_lang.Unexpanded.t
     ; allow_overlapping_dependencies : bool
     ; enabled_if : Blang.t
-    ; dune_version : Dune_lang.Syntax.Version.t
     }
 
   include Stanza.Make (struct
@@ -33,7 +33,7 @@ module Emit = struct
     let extension_field = extension in
     let module_systems =
       let module_system =
-        enum [ "es6", Melange.Module_system.Es6; "commonjs", CommonJs ]
+        enum [ "esm", Melange.Module_system.ESM; "es6", ESM; "commonjs", CommonJS ]
       in
       let+ module_systems =
         repeat
@@ -108,6 +108,7 @@ module Emit = struct
            (located (repeat Dep_conf.decode_no_files))
            ~default:(loc, [])
        and+ preprocess, preprocessor_deps = Preprocess.preprocess_fields
+       and+ lint = field "lint" Lint.decode ~default:Lint.default
        and+ promote = field_o "promote" Rule_mode_decoder.Promote.decode
        and+ instrumentation = Preprocess.Instrumentation.instrumentation
        and+ compile_flags = Ordered_set_lang.Unexpanded.field "compile_flags"
@@ -118,7 +119,7 @@ module Emit = struct
          let open Enabled_if in
          let allowed_vars = Any in
          decode ~allowed_vars ~since:None ()
-       and+ dune_version = Dune_lang.Syntax.get_exn Stanza.syntax in
+       in
        let preprocess =
          let init =
            let f libname = Preprocess.With_instrumentation.Ordinary libname in
@@ -137,27 +138,20 @@ module Emit = struct
        ; preprocess
        ; runtime_deps
        ; preprocessor_deps
+       ; lint
        ; promote
        ; compile_flags
        ; allow_overlapping_dependencies
        ; enabled_if
-       ; dune_version
        })
   ;;
 
   let target_dir (emit : t) ~dir = Path.Build.relative dir emit.target
 end
 
-let syntax =
-  Dune_lang.Syntax.create
-    ~name:Dune_project.Melange_syntax.name
-    ~desc:"the Melange extension"
-    [ (0, 1), `Since (3, 8) ]
-;;
-
 let () =
   Dune_project.Extension.register_simple
-    syntax
+    Dune_lang.Melange.syntax
     (return
        [ ( "melange.emit"
          , let+ stanza = Emit.decode in
