@@ -64,16 +64,16 @@ module Processed = struct
     }
 
   let dyn_of_config
-    { stdlib_dir
-    ; source_root
-    ; obj_dirs
-    ; src_dirs
-    ; hidden_obj_dirs
-    ; hidden_src_dirs
-    ; flags
-    ; extensions
-    ; indexes
-    }
+        { stdlib_dir
+        ; source_root
+        ; obj_dirs
+        ; src_dirs
+        ; hidden_obj_dirs
+        ; hidden_src_dirs
+        ; flags
+        ; extensions
+        ; indexes
+        }
     =
     let open Dyn in
     record
@@ -124,7 +124,7 @@ module Processed = struct
     type nonrec t = t
 
     let name = "merlin-conf"
-    let version = 6
+    let version = 7
     let to_dyn _ = Dyn.String "Use [dune ocaml dump-dot-merlin] instead"
 
     let test_example () =
@@ -178,20 +178,20 @@ module Processed = struct
   ;;
 
   let to_sexp
-    ~unit_name
-    ~opens
-    ~pp
-    ~reader
-    { stdlib_dir
-    ; source_root
-    ; obj_dirs
-    ; src_dirs
-    ; hidden_obj_dirs
-    ; hidden_src_dirs
-    ; flags
-    ; extensions
-    ; indexes
-    }
+        ~unit_name
+        ~opens
+        ~pp
+        ~reader
+        { stdlib_dir
+        ; source_root
+        ; obj_dirs
+        ; src_dirs
+        ; hidden_obj_dirs
+        ; hidden_src_dirs
+        ; flags
+        ; extensions
+        ; indexes
+        }
     =
     let make_directive tag value = Sexp.List [ Atom tag; value ] in
     let make_directive_of_path tag path =
@@ -214,26 +214,33 @@ module Processed = struct
       Path.Set.to_list_map hidden_src_dirs ~f:(make_directive_of_path "SH")
     in
     let flags =
-      let flags =
+      (* Order matters here. The flags should be communicated to Merlin in the
+         same order that they are passed to the compiler: user flags, pp flags
+         and then opens *)
+      let base_flags =
         match flags with
-        | [] -> []
+        | [] -> None
         | flags ->
-          [ make_directive "FLG" (Sexp.List (List.map ~f:(fun s -> Sexp.Atom s) flags)) ]
+          Some
+            (make_directive "FLG" (Sexp.List (List.map ~f:(fun s -> Sexp.Atom s) flags)))
       in
-      let flags =
+      let pp_flags =
         match pp with
-        | None -> flags
+        | None -> None
         | Some { flag; args } ->
-          make_directive "FLG" (Sexp.List [ Atom (Pp_kind.to_flag flag); Atom args ])
-          :: flags
+          Some
+            (make_directive "FLG" (Sexp.List [ Atom (Pp_kind.to_flag flag); Atom args ]))
       in
-      match opens with
-      | [] -> flags
-      | opens ->
-        make_directive
-          "FLG"
-          (Sexp.List (Ocaml_flags.open_flags opens |> List.map ~f:(fun x -> Sexp.Atom x)))
-        :: flags
+      let open_flags =
+        match opens with
+        | [] -> None
+        | opens ->
+          let open_flags =
+            Ocaml_flags.open_flags opens |> List.map ~f:(fun x -> Sexp.Atom x)
+          in
+          Some (make_directive "FLG" (Sexp.List open_flags))
+      in
+      List.filter_opt [ base_flags; pp_flags; open_flags ]
     in
     let unit_name = [ make_directive "UNIT_NAME" (Sexp.Atom unit_name) ] in
     let suffixes =
@@ -280,16 +287,16 @@ module Processed = struct
   ;;
 
   let to_dot_merlin
-    stdlib_dir
-    source_root
-    pp_configs
-    flags
-    obj_dirs
-    src_dirs
-    hidden_obj_dirs
-    hidden_src_dirs
-    extensions
-    indexes
+        stdlib_dir
+        source_root
+        pp_configs
+        flags
+        obj_dirs
+        src_dirs
+        hidden_obj_dirs
+        hidden_src_dirs
+        extensions
+        indexes
     =
     let b = Buffer.create 256 in
     let printf = Printf.bprintf b in
@@ -482,17 +489,17 @@ module Unprocessed = struct
     }
 
   let make
-    ~requires_compile
-    ~requires_hidden
-    ~stdlib_dir
-    ~flags
-    ~preprocess
-    ~libname
-    ~modules
-    ~obj_dir
-    ~dialects
-    ~ident
-    ~modes
+        ~requires_compile
+        ~requires_hidden
+        ~stdlib_dir
+        ~flags
+        ~preprocess
+        ~libname
+        ~modules
+        ~obj_dir
+        ~dialects
+        ~ident
+        ~modes
     =
     (* Merlin shouldn't cause the build to fail, so we just ignore errors *)
     let mode =
@@ -632,25 +639,25 @@ module Unprocessed = struct
   ;;
 
   let process
-    ({ modules
-     ; ident = _
-     ; config =
-         { stdlib_dir
-         ; extensions
-         ; readers
-         ; flags
-         ; objs_dirs
-         ; requires_compile
-         ; requires_hidden
-         ; preprocess = _
-         ; libname = _
-         ; mode
-         }
-     } as t)
-    sctx
-    ~dir
-    ~more_src_dirs
-    ~expander
+        ({ modules
+         ; ident = _
+         ; config =
+             { stdlib_dir
+             ; extensions
+             ; readers
+             ; flags
+             ; objs_dirs
+             ; requires_compile
+             ; requires_hidden
+             ; preprocess = _
+             ; libname = _
+             ; mode
+             }
+         } as t)
+        sctx
+        ~dir
+        ~more_src_dirs
+        ~expander
     =
     let open Action_builder.O in
     let+ config =
@@ -680,8 +687,12 @@ module Unprocessed = struct
              >>= function
              | Some lib ->
                let+ libs =
-                 let linking =
-                   Dune_project.implicit_transitive_deps (Scope.project scope)
+                 let* linking =
+                   let+ ocaml = Context.ocaml (Super_context.context sctx) in
+                   Dune_project.Implicit_transitive_deps.to_bool
+                     (Dune_project.implicit_transitive_deps
+                        (Scope.project scope)
+                        ocaml.version)
                  in
                  Lib.closure [ lib ] ~linking
                  |> Resolve.Memo.peek
