@@ -1,5 +1,6 @@
 open Import
 open Memo.O
+module Parallel_map = Memo.Make_parallel_map (Module_name.Map)
 
 module Common = struct
   module Encode = struct
@@ -73,7 +74,7 @@ module Stdlib = struct
   let map t ~f = { t with modules = Module_name.Map.map t.modules ~f }
 
   let traverse t ~f =
-    let+ modules = Module_name.Parallel_map.parallel_map t.modules ~f:(fun _ -> f) in
+    let+ modules = Parallel_map.parallel_map t.modules ~f:(fun _ -> f) in
     { t with modules }
   ;;
 
@@ -427,7 +428,7 @@ module Group = struct
       { t with alias; modules }
 
     and parallel_map_modules modules ~f =
-      Module_name.Parallel_map.parallel_map modules ~f:(fun _ n ->
+      Parallel_map.parallel_map modules ~f:(fun _ n ->
         match n with
         | Module m ->
           let+ m = f m in
@@ -438,8 +439,6 @@ module Group = struct
     ;;
   end
 
-  let group_interfaces (t : t) m = parents t m |> List.map ~f:lib_interface
-
   let make_alias_for t m ~parents =
     match Module.kind m with
     | Alias _ | Wrapped_compat -> []
@@ -719,7 +718,6 @@ module Wrapped = struct
 
   let find t name = Group.find t.group name
   let find_dep t ~of_ name = Group.find_dep t.group ~of_ name
-  let group_interfaces (t : t) m = Group.group_interfaces t.group m
   let alias_for t m = Group.alias_for t.group m
 end
 
@@ -923,6 +921,28 @@ let source_dirs =
     |> List.fold_left ~init:acc ~f:(fun acc f -> Path.Set.add acc (Path.parent_exn f)))
 ;;
 
+let compat_for_exn t m =
+  match t.modules with
+  | Singleton _ | Stdlib _ | Unwrapped _ -> assert false
+  | Wrapped { group; _ } ->
+    (match Module_name.Map.find group.modules (Module.name m) with
+     | None -> assert false
+     | Some (Module m) -> m
+     | Some (Group g) -> Group.lib_interface g)
+;;
+
+let entry_modules t =
+  List.filter
+    ~f:(fun m -> Module.visibility m = Public)
+    (match t.modules with
+     | Stdlib w -> Stdlib.lib_interface w |> Option.to_list
+     | Singleton m -> [ m ]
+     | Unwrapped m -> Unwrapped.entry_modules m
+     | Wrapped m ->
+       (* we assume this is never called for implementations *)
+       [ Wrapped.lib_interface m ])
+;;
+
 module With_vlib = struct
   type impl =
     { obj_map : Sourced_module.t Module_name.Unique.Map.t Lazy.t
@@ -1201,19 +1221,6 @@ module With_vlib = struct
     | Modules t -> { impl = fold t ~init ~f; vlib = [] }
   ;;
 
-  let compat_for_exn t m =
-    match t with
-    | Impl _ -> Code_error.raise "wrapped compat not supported for vlib" []
-    | Modules t ->
-      (match t.modules with
-       | Singleton _ | Stdlib _ | Unwrapped _ -> assert false
-       | Wrapped { group; _ } ->
-         (match Module_name.Map.find group.modules (Module.name m) with
-          | None -> assert false
-          | Some (Module m) -> m
-          | Some (Group g) -> Group.lib_interface g))
-  ;;
-
   let wrapped_compat t =
     match t with
     | Impl _ | Modules { modules = Stdlib _ | Singleton _ | Unwrapped _; _ } ->
@@ -1238,23 +1245,6 @@ module With_vlib = struct
     | Impl w -> Impl { w with impl = map w.impl }
   ;;
 
-  let entry_modules = function
-    | Impl i ->
-      Code_error.raise
-        "entry_modules: not defined for implementations"
-        [ "impl", dyn_of_impl i ]
-    | Modules t ->
-      List.filter
-        ~f:(fun m -> Module.visibility m = Public)
-        (match t.modules with
-         | Stdlib w -> Stdlib.lib_interface w |> Option.to_list
-         | Singleton m -> [ m ]
-         | Unwrapped m -> Unwrapped.entry_modules m
-         | Wrapped m ->
-           (* we assume this is never called for implementations *)
-           [ Wrapped.lib_interface m ])
-  ;;
-
   let wrapped = function
     | Modules t -> wrapped t
     | Impl { vlib = _; impl; _ } -> wrapped impl
@@ -1277,19 +1267,6 @@ module With_vlib = struct
          | Impl { impl; vlib = _; _ } -> alias_for impl m)
   ;;
 
-  let group_interfaces =
-    let group_interfaces t m =
-      match t.modules with
-      | Wrapped w -> Wrapped.group_interfaces w m
-      | Singleton w -> [ w ]
-      | _ -> []
-    in
-    fun t m ->
-      match t with
-      | Modules t -> group_interfaces t m
-      | Impl { impl; vlib; _ } -> group_interfaces impl m @ group_interfaces vlib m
-  ;;
-
   let local_open t m =
     alias_for t m
     |> List.map ~f:(fun m ->
