@@ -5,13 +5,13 @@ open Import
 val ocaml_flags
   :  Super_context.t
   -> dir:Path.Build.t
-  -> Ocaml_flags.Spec.t
+  -> Dune_lang.Ocaml_flags.Spec.t
   -> Ocaml_flags.t Memo.t
 
 val link_flags
   :  Super_context.t
   -> dir:Path.Build.t
-  -> Link_flags.Spec.t
+  -> Dune_lang.Link_flags.Spec.t
   -> Link_flags.t Memo.t
 
 val link_env : dir:Path.Build.t -> Link_flags.t Memo.t
