@@ -7,9 +7,11 @@ val template_file : Path.t -> Path.t
 
 (** Generate the contents of an opam file. [template] is the filename and
     contents of the template file. *)
-val generate :
-  Dune_project.t -> Package.t -> template:(Path.t * string) option -> string
+val generate : Dune_project.t -> Package.t -> template:(Path.t * string) option -> string
 
-val add_rules : Super_context.t -> Dune_project.t -> unit Memo.t
-
-val add_opam_file_rules : Super_context.t -> Dune_project.t -> unit Memo.t
+val gen_rules
+  :  Super_context.t Memo.t
+  -> dir:Path.Build.t
+  -> nearest_src_dir:Source_tree.Dir.t option
+  -> src_dir:Path.Source.t
+  -> Build_config.Gen_rules.Rules.t option
