@@ -6,7 +6,7 @@ module Map_reduce =
     (Memo)
     (Monoid.Product (Monoid.Union (Path.Set)) (Monoid.Union (Path.Set)))
 
-let files dir =
+let files_with_filter dir ~filter =
   let prefix_with, dir = Path.extract_build_context_dir_exn dir in
   Source_tree.find_dir dir
   >>= function
@@ -23,6 +23,7 @@ let files dir =
             Source_tree.Dir.filenames dir
             |> String.Set.to_list
             |> Path.Set.of_list_map ~f:(fun fn -> Path.relative path fn)
+            |> Path.Set.filter ~f:filter
           in
           let empty_directories =
             if Path.Set.is_empty files then Path.Set.singleton path else Path.Set.empty
@@ -31,3 +32,5 @@ let files dir =
     in
     Dep.Set.of_source_files ~files ~empty_directories, files
 ;;
+
+let files = files_with_filter ~filter:(Fun.const true)
