@@ -1,5 +1,6 @@
 open Import
 open Dune_lang.Decoder
+module Ocaml_flags = Dune_lang.Ocaml_flags
 
 type for_ =
   | Executable
@@ -41,8 +42,7 @@ let decode (for_ : for_) =
     | Some names ->
       let names = Ordered_set_lang.replace_standard_with_empty names in
       let flags = Option.value ~default:Ordered_set_lang.Unexpanded.standard flags in
-      Foreign.Stubs.make ~loc ~language ~names ~mode:Mode.Select.All ~flags
-      :: foreign_stubs
+      Foreign.Stubs.make ~loc ~language ~names ~flags :: foreign_stubs
   in
   let+ loc = loc
   and+ preprocess, preprocessor_deps = Preprocess.preprocess_fields
@@ -57,9 +57,10 @@ let decode (for_ : for_) =
       (Dune_lang.Syntax.since Stanza.syntax (2, 0)
        >>> repeat (located Foreign.Archive.decode))
   and+ extra_objects =
-    field_o
+    field
       "extra_objects"
       (Dune_lang.Syntax.since Stanza.syntax (3, 5) >>> Foreign.Objects.decode)
+      ~default:Foreign.Objects.empty
   and+ c_flags =
     only_in_library
       (field_o "c_flags" (use_foreign >>> Ordered_set_lang.Unexpanded.decode))
@@ -132,9 +133,10 @@ let decode (for_ : for_) =
   in
   let foreign_archives =
     let foreign_archives = Option.value ~default:[] foreign_archives in
-    if version < (2, 0)
-       && List.is_non_empty foreign_stubs
-       && Option.is_some self_build_stubs_archive
+    if
+      version < (2, 0)
+      && List.is_non_empty foreign_stubs
+      && Option.is_some self_build_stubs_archive
     then
       User_error.raise
         ~loc:self_build_stubs_archive_loc
@@ -156,7 +158,6 @@ let decode (for_ : for_) =
          the "lib" prefix, however, since standard linkers require it). *)
       | Some name -> (loc, Foreign.Archive.stubs name) :: foreign_archives)
   in
-  let extra_objects = Option.value ~default:Foreign.Objects.empty extra_objects in
   { loc
   ; preprocess
   ; preprocessor_deps
