@@ -4,6 +4,7 @@ type t =
   { loc : Loc.t
   ; package : Package.t
   ; mld_files : Ordered_set_lang.t
+  ; files : Install_entry.File.t list
   }
 
 include Stanza.Make (struct
@@ -17,6 +18,8 @@ let decode =
   fields
     (let+ package = Stanza_common.Pkg.field ~stanza:"documentation"
      and+ mld_files = Ordered_set_lang.field "mld_files"
+     and+ files = field_o "files" (repeat Install_entry.File.decode)
      and+ loc = loc in
-     { loc; package; mld_files })
+     let files = Option.value files ~default:[] in
+     { loc; package; mld_files; files })
 ;;
