@@ -1,5 +1,4 @@
 (** Dune sub-systems *)
-open! Dune_engine
 
 (** This module allows to define sub-systems. The aim is to define everything
     related to the sub-system, such as the parser for [dune] files, the metadata
@@ -15,8 +14,6 @@ open! Dune_engine
     framework. "ppx_inline_test" and "qtest" are examples of backends. An end
     point is any library that contains inline tests. *)
 
-open! Import
-
 include module type of struct
   include Sub_system_intf
 end
@@ -25,7 +22,7 @@ end
 
     - connect the parser to the [dune] file parser
 
-    - connect the metatada generator [M.to_sexp] so that metadata are included
+    - connect the metadata generator [M.to_sexp] so that metadata are included
       in installed [<lib>.dune] files *)
 module Register_backend (M : Backend) : Registered_backend with type t := M.t
 
@@ -34,8 +31,8 @@ module Register_backend (M : Backend) : Registered_backend with type t := M.t
     - connect the parser to the [dune] file parser
 
     - connect the rule generator to the rule generator for libraries *)
-module Register_end_point (M : End_point) : sig end
+module Register_end_point (_ : End_point) : sig end
 
 (** Scan the sub-systems used by the library and generate rules for all of the
     ones that needs it. *)
-val gen_rules : Library_compilation_context.t -> unit
+val gen_rules : Library_compilation_context.t -> unit Memo.t
