@@ -35,7 +35,6 @@ let add_stanza db ~dir (acc, pps) stanza =
   match Stanza.repr stanza with
   | Library.T l ->
     let+ lib =
-      let open Memo.O in
       let+ resolve =
         Lib.DB.resolve_when_exists db (l.buildable.loc, Library.best_name l)
       in
@@ -64,7 +63,6 @@ let add_stanza db ~dir (acc, pps) stanza =
        else acc, pps)
   | Executables.T exes ->
     let+ libs =
-      let open Memo.O in
       let* compile_info =
         let* scope = Scope.DB.find_by_dir dir in
         let dune_version =
@@ -72,24 +70,20 @@ let add_stanza db ~dir (acc, pps) stanza =
           Dune_project.dune_version project
         in
         let+ pps =
-          Resolve.Memo.read_memo
-            (Preprocess.Per_module.with_instrumentation
-               exes.buildable.preprocess
-               ~instrumentation_backend:
-                 (Lib.DB.instrumentation_backend (Scope.libs scope)))
+          Instrumentation.with_instrumentation
+            exes.buildable.preprocess
+            ~instrumentation_backend:(Lib.DB.instrumentation_backend (Scope.libs scope))
+          |> Resolve.Memo.read_memo
           >>| Preprocess.Per_module.pps
         in
-        let names = Nonempty_list.to_list exes.names in
-        let merlin_ident = Merlin_ident.for_exes ~names:(List.map ~f:snd names) in
         Lib.DB.resolve_user_written_deps
           db
-          (`Exe names)
+          (`Exe exes.names)
           exes.buildable.libraries
           ~pps
           ~dune_version
           ~allow_overlaps:exes.buildable.allow_overlapping_dependencies
           ~forbidden_libraries:exes.forbidden_libraries
-          ~merlin_ident
       in
       let+ available = Lib.Compile.direct_requires compile_info in
       Resolve.peek available
@@ -118,6 +112,7 @@ let libs_and_ppx_under_dir sctx ~db ~dir =
       Source_tree_map_reduce.map_reduce
         dir
         ~traverse:Source_dir_status.Set.all
+        ~trace_event_name:"Utop rules loading"
         ~f:(fun dir ->
           let dir =
             Path.Build.append_source
@@ -144,11 +139,58 @@ let requires ~loc ~db ~libs =
   >>= Lib.closure ~linking:true
 ;;
 
+let utop_dev_tool_lock_dir_exists =
+  Memo.Lazy.create (fun () ->
+    let path = Dune_pkg.Lock_dir.dev_tool_lock_dir_path Utop in
+    Fs_memo.dir_exists (Path.Outside_build_dir.In_source_dir path))
+;;
+
+let utop_findlib_conf = Filename.concat utop_dir_basename "findlib.conf"
+
+(* The lib directory of the utop package and of each of its dependencies within
+   the _build directory (or the toolchains directory in the case of the OCaml
+   compiler). *)
+let utop_ocamlpath = Memo.Lazy.create (fun () -> Pkg_rules.dev_tool_ocamlpath Utop)
+
+(* Creates a rule that generates a custom findlib.conf containing the path to
+   the utop library as well as all of its dependencies in the _build directory
+   (or the toolchains directory in the case of the OCaml compiler). Utop uses
+   findlib to locate libraries at runtime. When utop is running as a devtool,
+   libraries are not in the location suggested by the default findlib.conf
+   (there may not even be a default findlib.conf on the current system) and so
+   we need to tell findlib where to look for libraries by means of a custom
+   findlib.conf file. *)
+let findlib_conf sctx ~dir =
+  let* lock_dir_exists = Memo.Lazy.force utop_dev_tool_lock_dir_exists in
+  match lock_dir_exists with
+  | false ->
+    (* If there isn't lockdir don't create the findlib.conf rule. *)
+    Memo.return ()
+  | true ->
+    let path = Path.Build.relative dir utop_findlib_conf in
+    let* ocamlpath = Memo.Lazy.force utop_ocamlpath in
+    let findlib_path =
+      String.concat (ocamlpath |> List.map ~f:Path.to_absolute_filename) ~sep:":"
+    in
+    let action = Action_builder.write_file path (sprintf "path=\"%s\"" findlib_path) in
+    Super_context.add_rule sctx ~dir action
+;;
+
+let lib_db sctx ~dir =
+  let* scope = Scope.DB.find_by_dir dir in
+  let* lock_dir_exists = Memo.Lazy.force utop_dev_tool_lock_dir_exists in
+  match lock_dir_exists with
+  | false -> Memo.return (Scope.libs scope)
+  | true ->
+    let* ocamlpath = Memo.Lazy.force utop_ocamlpath in
+    Lib.DB.of_paths (Super_context.context sctx) ~paths:ocamlpath
+    >>| Lib.DB.with_parent ~parent:(Some (Scope.libs scope))
+;;
+
 let setup sctx ~dir =
-  let open Memo.O in
   let* expander = Super_context.expander sctx ~dir in
   let* scope = Scope.DB.find_by_dir dir in
-  let db = Scope.libs scope in
+  let* db = lib_db sctx ~dir in
   let* libs, pps = libs_and_ppx_under_dir sctx ~db ~dir:(Path.build dir) in
   let pps =
     if List.is_empty pps
@@ -191,19 +233,18 @@ let setup sctx ~dir =
       ~requires_link
       ~requires_compile:requires
       ~flags
-      ~js_of_ocaml:None
+      ~js_of_ocaml:(Js_of_ocaml.Mode.Pair.make None)
       ~melange_package_name:None
       ~package:None
       ~preprocessing
   in
+  let* () = findlib_conf sctx ~dir in
   let toplevel = Toplevel.make ~cctx ~source ~preprocess:pps expander in
   Toplevel.setup_rules toplevel ~linkage:Exe.Linkage.byte
 ;;
 
 let requires_under_dir sctx ~dir =
-  let open Memo.O in
-  let* scope = Scope.DB.find_by_dir dir in
-  let db = Scope.libs scope in
+  let* db = lib_db sctx ~dir in
   let* libs = libs_under_dir sctx ~db ~dir:(Path.build dir) in
   let loc = Toplevel.Source.loc (source ~dir) in
   requires ~loc ~db ~libs
