@@ -1,4 +1,3 @@
-module Execution_env = Execution_env
 module Log = Log
 module Persistent = Persistent
 module Report_error = Report_error
@@ -7,13 +6,10 @@ module Stringlike = Stringlike
 module type Stringlike = Stringlike_intf.S
 
 module Build_path_prefix_map = Build_path_prefix_map0
-module Flock = Flock
 module Global_lock = Global_lock
 module Action = Action
 module Alias_name = Alias_name
-module Terminal_signals = Terminal_signals
 module Gc = Gc
-module Prog = Prog
 open Stdune
 
 let xdg =
@@ -33,6 +29,6 @@ let xdg =
 let frames_per_second () =
   match Dune_config.Config.(get threaded_console_frames_per_second) with
   | `Custom fps -> fps
-  | `Default when Execution_env.inside_emacs -> 15
+  | `Default when Stdune.Execution_env.inside_emacs -> 15
   | `Default -> 60
 ;;
