@@ -11,6 +11,8 @@ type 'a t =
   ; man : 'a
   }
 
+val to_dyn : ('a -> Dyn.t) -> 'a t -> Dyn.t
+
 (** Compute the opam layout from prefix. the opam layout is used for _build *)
 val opam_from_prefix : 'a -> relative:('a -> string -> 'a) -> 'a t
 
@@ -18,6 +20,7 @@ val opam_from_prefix : 'a -> relative:('a -> string -> 'a) -> 'a t
 val complete : 'a option t -> 'a option t
 
 val map : f:('a -> 'b) -> 'a t -> 'b t
+val map2 : f:('a -> 'b -> 'c) -> 'a t -> 'b t -> 'c t
 
 (** return the roots of the first argument if present *)
 val first_has_priority : 'a option t -> 'a option t -> 'a option t
