@@ -8,9 +8,9 @@ open Stdune
     always returns [true]. In the file-watching mode, there may be multiple
     runs, separated by calls to [restart].
 
-    Upon [restart], all previously created [Run.t] values are reset, which
-    causes subsequent calls of [is_current] on these values to return [false]. *)
-type t
+    Upon [restart], all previously created [Run.t] values stop being current, so
+    the subsequent calls of [is_current] on these values will return [false]. *)
+type t [@@immediate]
 
 val to_dyn : t -> Dyn.t
 
@@ -20,5 +20,8 @@ val current : unit -> t
 (** Check whether this run is the current one. *)
 val is_current : t -> bool
 
+(** Compare runs: the current run is greater than earlier ones. *)
+val compare : t -> t -> Ordering.t
+
 (** End the current run and start a new one. *)
 val restart : unit -> unit
