@@ -15,10 +15,12 @@ open Dune_tests_common
 
 let () = init ()
 
-let foo_meta = {|
+let foo_meta =
+  {|
 requires = "bar"
 requires(ppx_driver) = "baz"
 |}
+;;
 
 let db_path : Path.Outside_build_dir.t =
   External (Path.External.of_filename_relative_to_initial_cwd "../unit-tests/findlib-db")
@@ -42,7 +44,7 @@ let findlib =
     ; natdynlink_supported = Dynlink_supported.By_the_os.of_bool true
     ; ext_dll = ".so"
     ; stdlib_dir = Path.source @@ Path.Source.(relative root) "stdlib"
-    ; ccomp_type = Other "gcc"
+    ; ccomp_type = Cc
     ; ocaml_version_string = "4.02.3"
     ; ocaml_version = Ocaml.Version.make (4, 14, 1)
     }
@@ -113,7 +115,8 @@ let%expect_test "configurator" =
 
 let%expect_test "builtins" =
   print_pkg_archives "str";
-  [%expect {|
+  [%expect
+    {|
     Available { byte = []; native = [] } |}];
   print_pkg_archives "dynlink";
   [%expect
